function code = dbsCloseFrame( dbs_des, channel_number, param_struct, varargin)
% Close Process for write frame data in channel.
%   code = dbsCloseFrame( dbs_des, channel_number, param_struct, ...)
%
%       dbs_des(numeric)    : opened dbStore's descriptor id
%       channel_number(numeric)
%                           : channel number
%       param_struct(struct): parameter of channel
%
%       code(numeric)       : status, 0 is success, other is error.
%
%  OPTIONS :
%       'verbose'           : If an error occurs, output messages to display.
%
%  Example :
%       des = dbsOpen( .... );
%       if des > 0
%          while true
%              ch=1;
%              code = dbsOpenFrame(des, ch, 640,480,'RGBA');
%              if code ~= 0 
%                 break;
%              end
%              code = dbsWriteFrame(des, ch, 1, frame_array); 
%              if code ~= 0 
%                 break;
%              end
% 
%              param=initParanList();
%              ....
%              code = dbsCloseFrame(des, ch, param); 
%              if code ~= 0 
%                 break;
%              end
%              ....
%          end
%          ....
%          dbsClsoe(des);
%       end
%
%  Note :
%    IF parameter is nothing, param_struct = struct([]).
%
%  Last uodate : 2012-05-11
%
%  See also
%      dbsOpen dbsWrite dbsClose dbsAbort dbsMessage dbstoreVersion
%      dbsOpenFrame dbsWriteFrame dbsWriteFrames dbsCloseFrame
%      initParamList setParam printParam dbstore.txt

if nargout == 1
    code = -999;
end
if nargin < 3
    disp('Input argument is short.');
    disp('Type "help dbsCloseFrame".');
    return;
end

ip = inputParser;

ip.addRequired('dbs_des', @(x)isnumeric(x) && isscalar(x) && x > 0);
ip.addRequired('channel_number', @(x)isnumeric(x) && isscalar(x) && x > 0);
ip.addRequired('param_struct', @(x)isstruct(x) );

ip.parse(dbs_des, channel_number, param_struct);
% 'verbose' 
arg_check = true;
err_args = '';
verbose = false;
nvarargin = length(varargin);
for idx = 1:nvarargin
    if isempty(varargin{idx})
        continue;
    elseif ~ischar(varargin{idx}) 
        err_args = strcat(err_args,[' arg#' num2str(idx)]);
        arg_check = false;
        continue;
    end
    switch varargin{idx}
        case 'verbose'
            verbose = true;
        otherwise
            arg_check = false;
            err_args = strcat(err_args,[' "' varargin{idx} '"']);
    end
end
if( ~arg_check ) 
    if verbose
        disp(['<Error: illegal arguments :' err_args ' >']);
        disp('Type "help dbsCloseFrame".');
    end
    return;
end
liblabcom = 'libdbstore';
if ~loadLibLabcom( liblabcom, verbose )
    code = -9998;
    return;
end
%
pcnt = length(param_struct);
pbytes = 1;
for pno = 1:pcnt
    pbytes = pbytes + length(param_struct(pno).Pname) + ...
        length(param_struct(pno).Pvalue) + 4;
end
plist_ptr = libpointer('voidPtr',zeros(pbytes, 1, 'int8'));
%
% DBSTORE_API unsigned int dbsSetParam2Buffer( void* buffer, unsigned int offset,
%		const char* pname, const char* pvalue, const char* ptype); // add 2010-11-19 for MATLAB
lib_func = 'dbsSetParam2Buffer';
offset = 0;
%
for pno = 1:pcnt
    offset = calllib( liblabcom, lib_func, plist_ptr, offset, ...
        param_struct(pno).Pname, param_struct(pno).Pvalue, num2str(param_struct(pno).Ptype));
end
%
% DBSTORE_API int dbsCloseFrame2(int dbs_d, int ch_no,
%			unsigned int plist_serial_bytes, void* plist_serial);
lib_func = 'dbsCloseFrame2';
code = calllib( liblabcom, lib_func, dbs_des, channel_number, pbytes, plist_ptr );
if code < 0
    if verbose
        dbsMessage( code, lib_func);
    end
end
end
