function des = dbsOpen( mail_address, data_name, shot_number, sub_shot, data_type, varargin)
% Open Process for dbStore.
%   des = dbsOpen( mail_address, data_name, shot_number, sub_shot, ...
%                  data_type, ...)
%
%       mail_address(text) : your mail address
%           When you delete the saved data, this is necessary.
%       data_name(text)    : name of data
%           This uses the data name which you applied for to LABCOM.
%       shot_number(numeric) : SHOT number
%       sub_shot(numeric)    : SHOT sub-number(It is usually 1.)
%       data_type(text)      : 'RAW' or 'ANA'
%           if the data is raw, it is 'RAW', other is 'ANA'
%
%       des : result code
%           0 <  : dbStore's descriptor id, use in other function.
%           0 => : error code.
%
%  OPTIONS :
%       'verbose' : If an error occurs, output messages to display.
%       'cndb_timeout',timeout_sec(numeric)
%                 : seconds of timeout for db connection.
%       'cndb_retry',retry(numeric)
%                 : count of retry for db connection.
%
%  Example :
%       des = dbsOpen('xxx@nifs.ac.jp', 'DATA_X', 1000, 1, 'RAW');
%       if des > 0 
%          disp('Success open');
%          ....
%          ....
%          dbsClose(des);
%       end
%
%  Last update : 2012-05-11
%
%  See also
%      dbsWrite dbsClose dbsAbort dbsMessage dbstoreVersion
%      dbsOpenFrame dbsWriteFrame dbsWriteFrames dbsCloseFrame dbstore.txt

if nargout == 1
    des = -999;
end
if nargin < 5 || nargout ~= 1  
   disp('Sytax error or input argument is short.');
   disp('Type "help dbsOpen".');
   return;
end

ip = inputParser;

ip.addRequired('mail_address', @ischar);
ip.addRequired('data_name', @ischar);
ip.addRequired('shot_number', @(x)isnumeric(x) && isscalar(x) && x > 0);
ip.addRequired('sub_shot', @(x)isnumeric(x) && isscalar(x) && x >= 0);
ip.addRequired('data_type', @ischar);

ip.parse(mail_address, data_name, shot_number, sub_shot, data_type);

arg_check = true;
err_args = '';
verbose = false;
cndb_timeout = 5;
flg_cndb_timeout = false;
cndb_retry = 3;
flg_cndb_retry = false;

nvarargin = length(varargin);
for idx = 1:nvarargin
    if isempty(varargin{idx})
        continue;
    elseif flg_cndb_timeout
        if isnumeric(varargin{idx}) && isscalar(varargin{idx})
            cndb_timeout = varargin{idx};
            flg_cndb_timeout = false;
            continue
        else
            arg_check = false;
            break
        end
    elseif flg_cndb_retry
        if isnumeric(varargin{idx}) && isscalar(varargin{idx})
            cndb_retry = varargin{idx};
            flg_cndb_retry = false;
            continue
        else
            arg_check = false;
            break
        end
    elseif ~ischar(varargin{idx}) 
        err_args = strcat(err_args,[' arg#' num2str(idx)]);
        arg_check = false;
        continue;
    end
    switch varargin{idx}
        case 'verbose'
            verbose = true;
        case 'cndb_timeout'
            flg_cndb_timeout = true;
        case 'cndb_retry'
            flg_cndb_retry = true;
        otherwise
            arg_check = false;
            err_args = strcat(err_args,[' "' varargin{idx} '"']);
    end
end
if( ~arg_check ) 
    if verbose
        disp(['<Error: illegal arguments :' err_args ' >']);
        disp('Type "help dbsOpen".');
    end
    return;
end
liblabcom = 'libdbstore';
if ~loadLibLabcom( liblabcom, verbose )
    des = -9998;
    return;
end
%
% DBSTORE_API int dbsOpen(const char *mail_address, const char *diag_name, 
%			 unsigned int shot_number, unsigned short sub_shot, short data_type );
% DBSTORE_API int dbsOpen2(const char *mail_address, const char *diag_name, 
%			 unsigned int shot_number, unsigned short sub_shot, short data_type,
%            int cndb_timeout, int cndb_retry);
if strcmp(data_type,'RAW')
    raw_ana_type = 1;
else
    raw_ana_type = 2;
end
lib_func = 'dbsOpen2';
des = calllib( liblabcom, lib_func, mail_address , data_name, ...
    shot_number, sub_shot, raw_ana_type, cndb_timeout, cndb_retry);
if des < 0
    if verbose
        dbsMessage( des, lib_func);
    end
end
end