function code = dbsOpenFrame( dbs_des, channel_number, frame_x, frame_y, image_type, varargin)
% Open Process for write frame data in channel.
%   code = dbsOpenFrame( dbs_des, channel_number, frame_x, frame_y, ...
%          image_type, ...)
%
%       dbs_des(numeric)    : opened dbStore's descriptor id
%       channel_number(numeric)
%                           : channel number
%       frame_x(numeric)    : x size of frame
%       frame_y(numeric)    : y size of frame
%       image_type(text)    : type of frame data.
%           Please refer to 'Note :' for the details.
%
%       code(numeric)       : status, 0 is success, other is error.
%
%  OPTIONS :
%       'accept_sizes'      : The size of frame data is accepted also except X*Y size. 
%       'verbose'           : If an error occurs, output messages to display.
%
%  Example :
%       des = dbsOpen( .... );
%       if des > 0
%          while true
%              ch=1;
%              code = dbsOpenFrame(des, ch, 640,480,'RGBA');
%              if code ~= 0 
%                 break;
%              end
%              code = dbsWriteFrame(des, ch, 1,frame_array); 
%              if code ~= 0 
%                 break;
%              end
% 
%              ....
%              code = dbsCloseFrame(des, ch, param); 
%              if code ~= 0 
%                 break;
%              end
%              ....
%          end
%          ....
%          if 0 == code 
%               dbsClsoe(des);
%          else
%               dbsAbort(des);
%          end
%       end
%
%  Note :
%      image_type(text) has the next kind.
%          'INT8'  : 8bit integer.
%          'INT16' : 16bit integer.
%          'INT32' : 32bit integer.
%          'INT64' : 64bit integer.
%          'FLT32' : single precision.
%          'FLT64' : double precision.
%          'GRAYn' : n bits gray image.(n:8-16) ex.) GRAY8
%          'GREYn' : same 'GRAYn'
%          'RGB'   : 24 bit color image.
%                    There is a meaning in letter order of 'R','G','B'
%          'RGBA'  : 32 bit color image.
%                    There is a meaning in letter order of 'R','G','B','A'
%
%  Last update : 2012-05-11
%
%  See also
%      dbsOpen dbsWrite dbsClose dbsAbort dbsMessage dbstoreVersion
%      dbsWriteFrame dbsWriteFrames dbsCloseFrame dbstore.txt

if nargout == 1
    code = -999;
end
if nargin < 5 
   disp('Input argument is short.');
   disp('Type "help dbsOpenFrame".');
   return;
end

ip = inputParser;

ip.addRequired('dbs_des', @(x)isnumeric(x) && isscalar(x) && x > 0);
ip.addRequired('channel_number', @(x)isnumeric(x) && isscalar(x) && x > 0);
ip.addRequired('frame_x', @(x)isnumeric(x) && isscalar(x) && x > 0);
ip.addRequired('frame_y', @(x)isnumeric(x) && isscalar(x) && x > 0);
ip.addRequired('image_type', @ischar);

ip.parse(dbs_des, channel_number, frame_x, frame_y, image_type);
% 'verbose' 
arg_check = true;
err_args = '';
verbose = false;
acceptsizes = 0;
nvarargin = length(varargin);
for idx = 1:nvarargin
    if isempty(varargin{idx})
        continue;
    elseif ~ischar(varargin{idx}) 
        err_args = strcat(err_args,[' arg#' num2str(idx)]);
        arg_check = false;
        continue;
    end
    switch varargin{idx}
        case 'verbose'
            verbose = true;
        case 'accept_sizes'
            acceptsizes = 1;
        otherwise
            arg_check = false;
            err_args = strcat(err_args,[' "' varargin{idx} '"']);
    end
end
if( ~arg_check ) 
    if verbose
        disp(['<Error: illegal arguments :' err_args ' >']);
        disp('Type "help dbsOpenFrame".');
    end
    return;
end
liblabcom = 'libdbstore';
if ~loadLibLabcom( liblabcom, verbose )
    code = -9998;
    return;
end
%
% DBSTORE_API int dbsOpenFrame(int dbs_d, int ch_no,
%			unsigned int x_size,unsigned int y_size, char* data_type );
% DBSTORE_API int dbsOpenFrame2(int dbs_d, int ch_no,
%			unsigned int x_size,unsigned int y_size, char* data_type, int sw_accept_size );
lib_func = 'dbsOpenFrame2';
code = calllib( liblabcom, lib_func, dbs_des, channel_number, ...
    frame_x, frame_y, image_type, acceptsizes);
if code < 0
    if verbose
        dbsMessage( code, lib_func);
    end
end
end
