function code = dbsWrite( dbs_des, channel_number, param_struct, data_array, varargin)
% Write channel data and parameter in dbStore.
%   code = dbsWrite( dbs_des, channel_number, param_struct, data_array, ...
%             varargin)
%
%       dbs_des(numeric)    : opened dbStore's descriptor id
%       channel_number(numeric)
%                           : channel number
%       param_struct(struct): struct array of channel's parameter
%           struct fields are Pname(char),Pvalue(char),Ptype(numeric).
%           describe it with 'struct([])' when there are not parameter to
%           register.
%       data_array(numeric,libpointer)
%                           : data array of channel
%           describe it with 'int16([])' when there are not data to 
%           register.
%
%       code(numeric)       : status, 0 is success, other is error.
%
%  OPTIONS :
%       'imagetype',image_type
%                           : The data type of channel data
%           Please refer to 'Note :' for the details.
%           When you do not give this option, it is decided from a model of
%           data_array.
%       'frames',frames     : number of the frames, 
%                             If you want to use frame constitution.
%       'verbose'           : If an error occurs, output messages to display.
%
%  Example :
%       des = dbsOpen('xxx@nifs.ac.jp', 'DATA_X', 213, 1, 'RAW');
%       if des > 0
%          pl = initParamList();
%          pl = setParam(pl, 'offset', 10.0);
%          pl = setParam(pl, 'gain', int32(2));
%          data = zeros(1000,1,'single');
%          code = dbsWrite( des, 1, pl, data,'verbose');
%          if code == 0 
%             dbsClose(des);
%          else
%             dbsAbort(des);
%          end
%       end
%
%  Note :
%      image_type(text) has the next kind.
%          'INT8'  : 8bit integer.
%          'INT16' : 16bit integer.
%          'INT32' : 32bit integer.
%          'INT64' : 64bit integer.
%          'FLT32' : single precision.
%          'FLT64' : double precision.
%          'GRAYn' : n bits gray image.(n:8-16) ex.) GRAY8
%          'GREYn' : same 'GRAYn'
%          'RGB'   : 24 bit color image.
%                    There is a meaning in letter order of 'R','G','B'
%          'RGBA'  : 32 bit color image.
%                    There is a meaning in letter order of 'R','G','B','A'
%
%  Last update 2012-8-23
%
%  See also
%      dbsOpen dbsClose dbsAbort initParamList setParam printParam
%      dbsOpenFrame dbsWriteFrame dbsWriteFrames dbsCloseFrame
%      dbsMessage dbstore.txt dbstoreVersion

if nargout == 1
    code = -999;
end
if nargin < 4
    disp('Sytax error or input argument is short.');
    disp('Type "help dbsWrite".');
    return;
end

ip = inputParser;

ip.addRequired('dbs_des', @(x)isnumeric(x) && isscalar(x) && x > 0);
ip.addRequired('channel_number', @(x)isnumeric(x) && isscalar(x) && x > 0);
ip.addRequired('param_strcut', @(x)isstruct(x) );

ip.parse(dbs_des, channel_number,param_struct);

% 'frames',frames(numeric)
% 'imageyype',image_type(char)
% 'verbose' 
arg_check = true;
err_args = '';
opt_frames = 0;
flg_frames = false;
opt_imagetype = '';
flg_imagetype = false;
verbose = false;
nvarargin = length(varargin);
for idx = 1:nvarargin
    if isempty(varargin{idx})
        continue;
    elseif flg_frames
        if (isnumeric(varargin{idx}) && isscalar(varargin{idx}) && varargin{idx} >= 0 )
            opt_frames = varargin{idx};
            flg_frames = false;
            continue
        else
            err_args = strcat(err_args,[' arg#' num2str(idx)]);
            arg_check = false;
            break
        end
    elseif flg_imagetype
        if ischar(varargin{idx}) 
            opt_imagetype = varargin{idx};
            flg_imagetype = false;
            continue
        else
            err_args = strcat(err_args,[' arg#' num2str(idx)]);
            arg_check = false;
            break
        end
    end
    if ~ischar(varargin{idx}) 
        err_args = strcat(err_args,[' arg#' num2str(idx)]);
        arg_check = false;
        continue;
    end    
    switch varargin{idx}
        case 'frames'
            flg_frames = true;
        case 'imagetype'
            flg_imagetype = true;
        case 'verbose'
            verbose = true;
        otherwise
            arg_check = false;
            err_args = strcat(err_args,[' ' varargin{idx}]);
    end
end
if( ~arg_check ) 
    if verbose
        disp(['<Error: illegal arguments :' err_args ' >']);
        disp('Type "help dbsWrite".');
    end
    return;
end

if isnumeric(data_array)
    data_ptr = libpointer([class(data_array) 'Ptr'], data_array);
elseif isa(data_array,'lib.pointer')
    data_ptr = data_array;
else
    if verbose
        disp('Input "data_array" argument is wrong.');
        disp('Type "help dbsWrite".');
    end
    return;
end
% if data is nothing(length(data)=0), it is data_ptr.isNull.
% if param_struct is nothing, length(param_struct) == 0.

if isempty(opt_imagetype)
    opt_imagetype = mClassPtr2ImageType(data_ptr.DataType);
end
%
% image_type checked in dll.
%
liblabcom = 'libdbstore';
if ~loadLibLabcom( liblabcom, verbose )
    code = -9998;
    return;
end

pcnt = length(param_struct);
if data_ptr.isNull
    data_bytes = 0; 
else
    data_bytes = numel(data_ptr.Value)*mClassPtr2bytes(data_ptr.DataType);
end

if opt_frames == 0 
    pbytes = 1;
    for pno = 1:pcnt
        pbytes = pbytes + length(param_struct(pno).Pname) + ...
            length(param_struct(pno).Pvalue) + 4;
    end
    plist_ptr = libpointer('voidPtr',zeros(pbytes,1,'int8'));
    %
    % DBSTORE_API unsigned int dbsSetParam2Buffer( void* buffer, unsigned int offset,
    %		const char* pname, const char* pvalue, const char* ptype); // add 2010-11-19 for MATLAB
    lib_func = 'dbsSetParam2Buffer';
    offset = 0;
    %
    for pno = 1:pcnt
        offset = calllib( liblabcom, lib_func, plist_ptr, offset, ...
            param_struct(pno).Pname, param_struct(pno).Pvalue, num2str(param_struct(pno).Ptype));
    end
    %
    % DBSTORE_API int dbsWrite5(int dbs_d, int ch_no, 
    %           unsigned int plist_serial_bytes, void* plist_serial,
    %			unsigned int data_length, void* data, char* data_type);
    lib_func = 'dbsWrite5';
    code = calllib( liblabcom, lib_func, dbs_des, channel_number, ...
        pbytes, plist_ptr, data_bytes, data_ptr ,opt_imagetype);
    if code < 0
        if verbose
            dbsMessage( code, lib_func);
        end
    end
else
    while true
        if data_bytes == 0 
            code = -27;
            break;
        end
        frame_x = 0;
        frame_y = 0;
        for pno = 1:pcnt
            if strcmp(param_struct(pno).Pname,'AcquisitionWindowWidth')
                frame_x = str2double(param_struct(pno).Pvalue);
            elseif strcmp(param_struct(pno).Pname,'AcquisitionWindowHeight')
                frame_y = str2double(param_struct(pno).Pvalue);
            end
        end
        if frame_x == 0 || frame_y == 0 
            code = -205;
            break;
        end
        opt_args = cell(1);
        if verbose
            opt_args{1} = 'verbose';
        end
        code = dbsOpenFrame( dbs_des, channel_number, frame_x, frame_y, opt_imagetype, opt_args);
        if code < 0
            break;
        end
        code = dbsWriteFrames( dbs_des, channel_number, 1, opt_frames, data_ptr, opt_args);
        if code < 0
            break;
        end
        code = dbsCloseFrame( dbs_des, channel_number, param_struct, opt_args);
        if code < 0
            break;
        end
    end
    if code == -999 
        if verbose
            disp('<Error: illegal argument >');
            disp('Type "help dbsWrite".');
        end
    end
end
end

function image_type = mClassPtr2ImageType( class_name )
% Convert name of ClassPtr to name of image type. 
%   image_type = mClassPtr2ImageType( class_name )
%

switch class_name
    case {'int8Ptr','uint8Ptr','voidPtr'}
        image_type = 'INT8';
    case {'int16Ptr','uint16Ptr' }
        image_type ='INT16';
    case {'int32Ptr','uint32Ptr'}
        image_type = 'INT32';
    case {'int64Ptr','uint64Ptr'}
        image_type = 'INT64';
    case {'singlePtr'}
        image_type = 'FLT32';
    case {'doublePtr'}
        image_type = 'FLT64';
    otherwise
        image_type = 'Unknown';
end
end