function code = dbsWriteFrame( dbs_des, channel_number, frame_number, frame_array, varargin)
% Write a frame data in channel.
%   code = dbsWriteFrame( dbs_des, channel_number, frame_number, ...
%          frame_array, ...)
%
%       dbs_des(numeric)    : opened dbStore's descriptor id
%       channel_number(numeric)
%                           : channel number
%       frame_number(numeric)
%                           : frame number
%       frame_array(numeric,libpointer)
%                           : data of a frame
%
%       code(numeric)       : status, 0 is success, other is error.
%
%  OPTIONS :
%       'verbose'           : If an error occurs, output messages to display.
%
%  Example :
%       des = dbsOpen( .... );
%       if des > 0
%          while true
%              ch=1;
%              code = dbsOpenFrame(des, ch, 640,480,'RGBA');
%              if code ~= 0 
%                 break;
%              end
%              frame_array=zeros(640,480,'uint32');
%               ... or frame_array=libpointer('uint32Ptr',zeros(640,480));
%              code = dbsWriteFrame(des, ch, 1,frame_array); 
%              if code ~= 0 
%                 break;
%              end
% 
%              ....
%              code = dbsCloseFrame(des, ch, param); 
%              if code ~= 0 
%                 break;
%              end
%              ....
%          end
%          ....
%          dbsClsoe(des);
%       end
%
%  Last update : 2012-5-11
%
%  See also
%      dbsOpen dbsWrite dbsClose dbsAbort dbsMessage dbstoreVersion
%      dbsOpenFrame dbsWriteFrames dbsCloseFrame dbstore.txt

if nargout == 1
    code = -999;
end
if nargin < 4
    disp('Input argument is short.');
    disp('Type "help dbsWriteFrame".');
    return;
end

ip = inputParser;

ip.addRequired('dbs_des', @(x)isnumeric(x) && isscalar(x) && x > 0);
ip.addRequired('channel_number', @(x)isnumeric(x) && isscalar(x) && x > 0);
ip.addRequired('frame_number', @(x)isnumeric(x) && isscalar(x) && x > 0);

ip.parse(dbs_des, channel_number, frame_number);
% 'verbose' 
arg_check = true;
err_args = '';
verbose = false;
nvarargin = length(varargin);
for idx = 1:nvarargin
    if isempty(varargin{idx})
        continue;
    elseif ~ischar(varargin{idx}) 
        err_args = strcat(err_args,[' "arg#' num2str(idx) '"']);
        arg_check = false;
        continue;
    end
    switch varargin{idx}
        case 'verbose'
            verbose = true;
        otherwise
            arg_check = false;
            err_args = strcat(err_args,[' "' varargin{idx} '"']);
    end
end
if( ~arg_check ) 
    if verbose
        disp(['<Error: illegal arguments :' err_args ' >']);
        disp('Type "help dbsWriteFrame".');
    end
    return;
end

if isnumeric(frame_array)
%    info = whos('data');
%    data_bytes = info.bytes;
    data_ptr = libpointer([ class(frame_array) 'Ptr'], frame_array);
elseif isa(frame_array,'lib.pointer')
    data_ptr = frame_array;
else
    if verbose
        disp('Input "frame_array" argument is wrong.');
        disp('Type "help dbsWriteFrame".');
    end
    return;
end
liblabcom = 'libdbstore';
if ~loadLibLabcom( liblabcom, verbose )
    code = -9998;
    return;
end
%
% DBSTORE_API int dbsWriteFrame(int dbs_d, int ch_no, int frame_no,
%			unsigned int data_length, void* data);
if data_ptr.isNull 
    data_bytes = 0;
else
    data_bytes = numel(data_ptr.Value)*mClassPtr2bytes(data_ptr.DataType);
end
lib_func = 'dbsWriteFrame';
code = calllib( liblabcom, lib_func, dbs_des, channel_number, ...
    frame_number, data_bytes, data_ptr );
if code < 0
    if verbose
        dbsMessage( code, lib_func);
    end
end
end
    