function code = dbsWriteFrames( dbs_des, channel_number, start_frame_number, ...
    frames, frames_array, varargin)
% Write frame data in channel.
%   code = dbsWriteFrames( dbs_des, channel_number, start_frame_number, ...
%          frames, frames_array, ...)
%
%       dbs_des(numeric)    : opened dbStore's descriptor id
%       channel_number(numeric)
%                           : channel number
%       start_frame_number(numeric)
%                           : start number of the frame
%       frames(numeric)     : number of the frames
%       frame_array(numeric,libpointer)
%                           : data of the frames
%
%       code(numeric)       : status, 0 is success, other is error.
%
%  OPTIONS :
%       'verbose'           : If an error occurs, output messages to display.
%
%  Example :
%       des = dbsOpen( .... );
%       if des > 0
%          while true
%              ch=1;
%              code = dbsOpenFrame(des, ch, 640,480,'RGBA');
%              if code ~= 0 
%                 break;
%              end
%              num_f = 30;
%              frame_array=zeros(640,480,num_f,'uint32');
%               ... or frame_array=libpointer('uint32Ptr',zeros(640,480,num_f));
%              code = dbsWriteFrames(des, ch, 1,num_f,frame_array); 
%              if code ~= 0 
%                 break;
%              end
% 
%              ....
%              code = dbsCloseFrame(des, ch, param); 
%              if code ~= 0 
%                 break;
%              end
%              ....
%          end
%          ....
%          dbsClsoe(des);
%       end
%
%  Last update : 2012-05-11
%
%  See also
%      dbsOpen dbsWrite dbsClose dbsAbort dbsMessage dbstoreVersion
%      dbsOpenFrame dbsWriteFrame dbsCloseFrame dbstore.txt

if nargout == 1
    code = -999;
end
if nargin < 5
    disp('Input argument is short.');
    disp('Type "help dbsWriteFrames".');
    return;
end

ip = inputParser;

ip.addRequired('dbs_des', @(x)isnumeric(x) && isscalar(x) && x > 0);
ip.addRequired('channel_number', @(x)isnumeric(x) && isscalar(x) && x > 0);
ip.addRequired('start_frame_number', @(x)isnumeric(x) && isscalar(x) && x > 0);
ip.addRequired('frames', @(x)isnumeric(x) && isscalar(x) && x > 0);

ip.parse(dbs_des, channel_number, start_frame_number, frames);
% 'verbose' 
arg_check = true;
err_args = '';
verbose = false;
nvarargin = length(varargin);
for idx = 1:nvarargin
    if isempty(varargin{idx})
        continue;
    elseif ~ischar(varargin{idx}) 
        err_args = strcat(err_args,[' arg#' num2str(idx)]);
        arg_check = false;
        continue;
    end
    switch varargin{idx}
        case 'verbose'
            verbose = true;
        otherwise
            arg_check = false;
            err_args = strcat(err_args,[' "' varargin{idx} '"']);
    end
end
if( ~arg_check ) 
    if verbose
        disp(['<Error: illegal arguments :' err_args ' >']);
        disp('Type "help dbsWriteFrames".');
    end
    return;
end
if isnumeric(frames_array)
%    info = whos('frames_array');
%    data_bytes = info.bytes;
    data_ptr = libpointer([class(frames_array) 'Ptr'], frames_array);
elseif isa(frames_array,'lib.pointer')
    data_ptr = frames_array;
else
    if verbose
        disp('Input argument is wrong.');
        disp('Type "help dbsWriteFrames".');
    end
    return;
end
liblabcom = 'libdbstore';
if ~loadLibLabcom( liblabcom, verbose )
    code = -9998;
    return;
end
%
% DBSTORE_API int dbsWriteFrames(int dbs_d, int ch_no, int start_frame_no,int num_frame,
%			unsigned int data_length, void* data);
%
if data_ptr.isNull
    data_bytes = 0;
else
    data_bytes = numel(data_ptr.Value)*mClassPtr2bytes(data_ptr.DataType);
end
lib_func = 'dbsWriteFrames';
code = calllib( liblabcom, lib_func, dbs_des, channel_number, ...
    start_frame_number, frames, data_bytes, data_ptr );
if code < 0
    if verbose
        dbsMessage( code, lib_func);
    end
end
end
    