function [pvalue, tf_code] = getParam( param_name, param_lists, cindex)
% Get Parameter value with the name of parameter in retrieved parameters.
%   [pvalue tf_code] = getParam( param_name, params_lists [, cindex])
%
%       param_name(text)        : parameter name
%       param_lists(cell,struct): retrieved channels parameters
%       cindex                  : index of params_cell array
%
%       pvalue(...)             : value of parameter
%       tf_code(logical)        : ture is success, false is error.
%
% Examples :
%   >> [data param] = retrieve('SX8O',98000,1,2);
%   ....
%   >> value = getParam('Shot',param)
%      value =
%              98000
%
%   >> [data param] = retrieve('SX8O',98000,1,0,'silent');
%   >> value = getParam('ChannelNumber',param,11)
%      value =
%              11
%
% Last update : 2012-05-11
%
% See also
%     printParam retrieve retrieve_t retrieveVersion

if nargout > 0 
    pvalue = 0;
end
if nargout > 1 
    tf_code = false;
end

if nargin < 2 || ~(iscell(param_lists) || isstruct(param_lists))
   disp('Input argument is short or illegal.');
   disp('Type "help getParam".');
   return;
end

if iscell(param_lists)
    if ~exist('cindex','var')
        cindex = 1;
    end
    c_num = length(param_lists);
    if c_num < 1 || c_num < cindex
        return;
    end
    
    pno_num = length(param_lists{cindex});
    for pno = 1:pno_num
        if strcmp(param_lists{cindex}(pno).Pname, param_name)
            pvalue = mRetrieveParamValue(param_lists{cindex}(pno).Pvalue, ...
                param_lists{cindex}(pno).Ptype);
            tf_code = true;
            break;
        end
    end
else
    pno_num = length(param_lists);
    for pno = 1:pno_num
        if strcmp(param_lists(pno).Pname, param_name)
            pvalue = mRetrieveParamValue(param_lists(pno).Pvalue, ...
                param_lists(pno).Ptype);
            tf_code = true;
            break;
        end
    end
end
end

function   pvalue = mRetrieveParamValue(param_value, param_type)
% Convert string-value to value with type.
%   pvalue = mRetrieveParamValue(param_value, param_type)
% 

switch param_type
    case 1  % string
        pvalue = param_value;
    case 2  % 8bit integer
        pvalue = str2num(['int8(' param_value ')']);
    case 3  % 16bit integer
        pvalue = str2num(['int16(' param_value ')']);
    case 4  % 32bit integer
        pvalue = str2num(['int32(' param_value ')']);
    case 5  % 32bit float
        pvalue = single(str2double(param_value));
    case 6  % 64bit double
        pvalue = str2double(param_value);
    case 7  % 64bit integer
        pvalue = str2num(['int64(' param_value ')']);
    otherwise
        pvalue = param_value;
end
end
