function [data_cell, param_cell, code, time_cell, tpara_cell] = ... 
    retrieve(diag_name, shot_number, sub_shot, channels, varargin)
% Retrieve shot data with diagnostics,shot#,subshot# and channel#.
%   [data_cell param_cell code time_cell tpara_cell] = ... 
%      retrieve( diag_name, shot_number, sub_shot, channels, ... )
%
%       diag_name(text)      : name of diagnostics
%       shot_number(numeric) : SHOT number
%       sub_shot(numeric)    : SHOT sub-number
%       channels
%          (numeric [vector]): channel number(s)
%          If it wants to retrieve all channels, it is 0. 
%          If it wants to retrieve a channel, it is N. 
%          If it wants to retrieve a range, it is  START#:END#.
%                                              (e.g.  [1,3,5:7] )
%          (text)            : signal tag name (e.g. 'sig-tag1')
%
%       data_cell(cell)     : data for each channels
%          If 'pointer' option , the type of data is libpointer.
%       param_cell(cell)    : parameter for each channels
%       code(numeric)       : result code
%          If it is 0 , this function is success. otherwise, this
%          function is error.
%       time_cell(cell)     : time array for each channels
%          If 'pointer' option , the type of time data is libpointer.
%       tpara_cell(cell)    : time's parameter for each channels
%
% Options :
%         'frame',frames(numeric [vector])
%                                : frame number
%             The format is the same as an channels parameter.
%         'host',hostname(text)  : retrieve from a direct server
%         'path',pathname(text)  : retrieve from a direct path
%         'wait'                 : wait for the appearance of data
%             It can't be used with 'et','tt'.'ss'.
%         'timeout',timeout_sec(numeric)
%                                : seconds of timeout for wait
%             The default is 180 sec.
%         'v4'                   : Convert to voltage or real number
%                                  (single precision)
%         'vflt'                 : same 'v4'
%         'v8'                   : Convert to voltage or real number
%                                  (double precision)
%         'vdbl'                 : same 'v8'
%         'silent'               : do not display messages
%         'pointer'              : a cell in data_cell have libpointer
%         'dataclass',classname(text)
%                                : fix the class of data.
%             'intN','uintN','single','double' (N=8,16,32,64)
%         'time-single'           : time data is a single type.
%
% Range Options :
%         'et',range(text)       : range by experiment timming
%             'N:M[u]' N:begin time, M:end time, u:unit(s,ms,us)(e.g. '1:2.5s' )
%         'tt',range(text)       : range by base of triggered time
%              format is same of 'et' option
%         'ss',range(text)       : range by sample number
%             'N:M' N:begin number, M:end number(e.g. '10000:20000' )
%
% Examples :
%    [data param code] = retrieve('SX8O',98000,1,0,'silent');
%    if code ~= 0
%       disp('Error');
%    else
%       printParam(param);
%    end
%
%
%    [data,~,code] = retrieve( 'SX8O', 98000,1,0,'wait','timeout',30 );
%    if code == 0
%       disp('Success');
%    end
%
%
%    >> [data,params,code,ta,tp] = retrieve('RADH',98000,1,[2:6,12],'v4','et','3:6s');
%    >> plot(ta{1},data{1});
%    >> printParam(params{1});
%
% Last update : 2020-05-20
%
% See also
%     printParam getParam retrieve_t changeSite currentSite retrieveVersion

%
% datasize = 0 : isempty(data{1}) is true.
%
ip = inputParser;
ip.KeepUnmatched = true;

ip.addRequired('diag_name', @ischar);
ip.addRequired('shot_number', @(x)isnumeric(x) && isscalar(x) && x > 0);
ip.addRequired('sub_shot',@(x)isnumeric(x) && isscalar(x) && x >= 0);

% ip.addRequired('channels',@(x)isnumeric(x) && all(x>=0) );
% ip.addParamValue('frame',0,@(x)isnumeric(x) && x >= 0);

%ip.parse(diag_name, shot_number, sub_shot, channels);
ip.parse(diag_name, shot_number, sub_shot);

if nargout > 0
    data_cell = cell(0);
end
if nargout > 1
    param_cell = cell(0);
end
if nargout > 3
    time_cell = cell(0);
end
if nargout > 4
    tpara_cell = cell(0);
end

arg_check = true;
verbose = true;
wait_time_sec = 0;
flg_timeout = false;
opt_host = '';
flg_host = false;
opt_path = 'ShotDataFS';    % add 2020-05-20
flg_path = false;
opt_frame = 0;
flg_frame = false;
nvarargin = length(varargin);
opt_vcnv = 0;       % 0=none 1=flt 2=dbl
opt_pointer = false;
err_arg = '';
opt_dataclass = '';
flg_dataclass = false;
is_time_double = 1;
opt_range = '';
opt_range_type = 0;
flg_range = false;

for idx = 1:nvarargin
    if isempty(varargin{idx})
        continue;
    elseif flg_timeout
        if isnumeric(varargin{idx}) && isscalar(varargin{idx})
            wait_time_sec = varargin{idx};
            flg_timeout = false;
            continue
        else
            arg_check = false;
            break
        end
    elseif flg_frame
        if isnumeric(varargin{idx}) && all(varargin{idx} >= 0)
            opt_frame = varargin{idx};
            flg_frame = false;
            continue
        else
            arg_check = false;
            break
        end
    elseif flg_dataclass
        if ischar(varargin{idx}) && any(strcmp(varargin{idx}, ...
                {'int8','int16','int32','int64',...
                 'uint8','uint16','uint32','uint64','single','double'}))
            opt_dataclass = varargin{idx};
            flg_dataclass = false;
            continue
        else
            arg_check = false;
            break
        end
    elseif flg_range
        if isempty(opt_range)
            opt_range = varargin{idx};
            flg_range = false;
            continue
        else
            arg_check = false;
            break
        end
      
    end
    
    if ~ischar(varargin{idx}) 
        err_arg = strcat(err_arg,[' arg#' num2str(idx+4)]);
        arg_check = false;
        break
    end
    if flg_host
        opt_host = varargin{idx};
        flg_host = false;
        continue
    elseif flg_path
        opt_path = varargin{idx};
        flg_path = false;
        continue
    end
    switch varargin{idx}
        case 'frame'
            flg_frame = true;
        case 'host'
            flg_host = true;
        case 'path'
            flg_path = true;
        case 'timeout'
            flg_timeout = true;
        case 'silent'
            verbose = false;
        case 'wait'
            if wait_time_sec == 0
                wait_time_sec = 180;
            end
        case 'vflt'
            opt_vcnv = 1;
        case 'vdbl'
            opt_vcnv = 2;
        case 'v4'
            opt_vcnv = 1;
        case 'v8'
            opt_vcnv = 2;
        case 'pointer'
            opt_pointer = true;
        case 'dataclass'
            flg_dataclass = true;
        case 'time-single'
            is_time_double = 0;
        case 'et'
            flg_range = true;
            opt_range_type=1;
        case 'tt'
            flg_range = true;
            opt_range_type=2;
        case 'ss'
            flg_range = true;
            opt_range_type=3;
        otherwise
            arg_check = false;
            err_arg = strcat(err_arg, [' ' varargin{idx}]);
    end
end
if not(ischar(channels) || ( isnumeric(channels) && all(channels>=0) ) )
    if verbose
        disp('<Error: illegal channles>');
        disp('Type "help retrieve".');
    end
    code = -2;
    return;
end

if( ~arg_check )
    if verbose
        disp(['<Error: illegal argument ' err_arg ' >']);
        disp('Type "help retrieve".');
    end
    code = -2;
    return;
end
if ~loadLibLabcom( 'libretrieve', verbose )
    code = -9;  % System error
    return;
end
[rtd, index_server] = ...
mRetrieveOpen( diag_name, shot_number, sub_shot, opt_host, opt_path, wait_time_sec, opt_range_type, opt_range, verbose);
if rtd < 0
    code=rtd;
    return;
end
%
[code, last_channel_number, syy, smm, sdd, shour, smin, ssec, trans_server] = ...
mRetrieveShotInfo( rtd, verbose);
if code < 0
    return
end

if ischar(channels)
    cnt_chs = 1;
else
    if channels(1) == 0
        channels = 1:last_channel_number;
    end
    cnt_chs = length(channels);
end
if verbose 
	fprintf('---Arc Shot Infomation---\n');
	fprintf('Index Server Name  = %s\n' ,char(index_server)    );
	fprintf('Trans Server Name  = %s\n' ,char(trans_server)    );
	fprintf('Shot Number        = %d\n' ,shot_number           );
 	fprintf('Sub Shot Number    = %d\n' ,sub_shot              );
 	fprintf('Channel            = %d\n' ,last_channel_number    );
	fprintf('DATE               = %4d-%2d-%2d\n',syy, smm, sdd  );
	fprintf('TIME               = %4d:%2d:%2d\n',shour, smin, ssec );
    if ischar(channels)
	fprintf('Request Channel    = %s\n'    ,channels);
    else
	fprintf('Request Channel    = %3d:%3d\n'    ,min(channels), max(channels) );
    end
end
if not(ischar(channels)) && max(channels) > last_channel_number
    code = -2;
    if verbose
		fprintf('<Error: Illegal channel number!>\n');
    end
    mRetrieveClose(rtd);
    return
end
data_cell = cell(cnt_chs,1);
if nargout > 1
    param_cell = cell(cnt_chs,1);
end 
data_idx = 0;
for tmp_x = 1:cnt_chs
    data_idx = data_idx+1;
    if not(ischar(channels)) 
        ch_no = channels(data_idx); 
        [code,data_bytes,comp_bytes,param_count,data_type,image_type,value_len,is_nframe,ch_comment] = ...
            mRetrieveChInfo( rtd, ch_no, verbose);
    else
        [code,data_bytes,comp_bytes,param_count,data_type,image_type,value_len,is_nframe,ch_comment,ch_no] = ...
            mRetrieveChInfo2( rtd, channels, verbose);
    end
    if code < 0
        return;
    end
    % 
    % if image_type is undefined, image_type remake with value_len
    if strcmp(image_type, '' )
        image_type = mValueLen2ImageType(value_len);
    end
    if verbose
        if 2 == data_type
            mes_type = 'ANA';
        else
            mes_type = 'RAW';
        end
		fprintf('---Channel Information---\n');
		fprintf('Channel Number     = %d\n'  ,ch_no         );
		fprintf('Data Type          = %s\n'  ,mes_type      );
		fprintf('Data Size          = %ld\n'  ,data_bytes    );
		fprintf('Comment            = %s\n'  ,ch_comment    );
        if 0 ~= comp_bytes
            fprintf('Compressed Size    = %ld\n'  ,comp_bytes);
        end
    end
    if ( is_nframe == 0 && opt_vcnv == 0 )
       [code,data_ptr,data_length] = ...
                mRetrieveChData( rtd, ch_no, data_bytes, image_type, opt_dataclass, verbose);
        if code < 0 
            break;
        else
            if opt_pointer
                data_cell{data_idx} = data_ptr;
            elseif data_length > 0 
                data_cell{data_idx} = data_ptr.Value;
            end
        end
    elseif ( is_nframe == 0 && opt_vcnv ~= 0 )
        [code,data_ptr,data_length] = ...
            mRetrieveChV( rtd, ch_no, data_bytes, image_type, opt_vcnv, verbose);
        if code < 0 
            break;
        else
            if opt_pointer
                data_cell{data_idx} = data_ptr;
            elseif data_length > 0 
                data_cell{data_idx} = data_ptr.Value;
            end
        end
      
    elseif ( is_nframe ~= 0 && opt_vcnv == 0 )
        if ~all( opt_frame <= is_nframe )
            code = -99;
            if verbose
                fprintf('<Error: not supported Volts convert with frame data!>\n');
            end
            break;
        end
        if opt_frame(1) == 0
            [code,begin_no,end_no] = mRetrieveRetrieveNo(rtd, ch_no);
            if( 0 == code )
                frame_nums = begin_no:end_no;
            else
                frame_nums = 1:is_nframe;
            end
        else
            frame_nums = opt_frame;
        end
        [code,data_ptr,data_length,frame_x,frame_y] = ...
            mRetrieveChFrameData( rtd, ch_no, frame_nums, verbose);
        if code == 1
            code = -99;
            if verbose
                fprintf('<Error: Each frame sizes differ. !>\n');
            end
            mRetrieveClose(rtd);
            return
        end
        if code < 0 
            break;
        else
            if opt_pointer
                data_cell{data_idx} = data_ptr;
            elseif data_length > 0
                f_num = length(frame_nums);
                if 1 == f_num && data_length ~= (frame_x*frame_y)
                    data_cell{data_idx} = data_ptr.Value;
                else
                    data_cell{data_idx} = reshape(data_ptr.Value,frame_x,frame_y, f_num);
                end
            end
        end
    else
        code = -99;
        if verbose
            fprintf('<Error: not supported Volts convert with frame data!>\n');
        end
        mRetrieveClose(rtd);
        return
    end
    if 1 > data_length
        if verbose
            fprintf('<Warning : CH#%d Data size is 0 !>\n',ch_no);
        end
    end
    if nargout < 2 % data only
        continue;
    end
    [code,param_cell{data_idx}] = ...
        mRetrieveChParams( rtd, ch_no, param_count, verbose);
    if code < 0
        break;
    end
    if nargout < 4 
        continue;
    end
    [code,time_ptr,data_length,params] = ...
    mRetrieveChTime( rtd, diag_name, shot_number, sub_shot, ch_no, ...
        is_time_double, index_server, verbose);
    if code < 0 
       break;
    else
       if opt_pointer
          time_cell{data_idx,1} = time_ptr;
       elseif data_length > 0 
          time_cell{data_idx,1} = time_ptr.Value;
       end
    end
    if nargout < 5 
        continue;
    end
    tpara_cell{data_idx,1} = params;
end

mRetrieveClose(rtd, false);
return
end

function [rtd, index_server] = ...
    mRetrieveOpen( diag_name, shot_number, sub_shot, host, path, wait_time_sec, range_type, range_str, verbose)
% Open process for retrieve-Protocol
%   [rtd, index_server] = ...
%        mRetrieveOpen( diag_name, shot_number, sub_shot, host, path, ...
%        wait_time_sec, verbose)
%
% 

%
%RETRIEVE_API int retrieveOpenWait( const char *diag_name ,const char* index_server ,
%							   unsigned int shot_number, unsigned short sub_shot ,int timeout_sec );
%       if timeout_sec = 0 is retrieveOpen().
%
%RETRIEVE_API int retrieveOpenDirectWait( const char *diag_name ,const char* server ,const char* path,
%							   unsigned int shot_number, unsigned short sub_shot ,int port ,int timeout_sec );
%
%RETRIEVE_API int retrieveOpenWaitWithRange( const char *diag_name ,const char* index_server,
%							   unsigned int shot_number, unsigned short sub_shot ,int timeout_sec,
%							   int range_type, const char* range_str);
% [int32, cstring, cstring, cstring]	retrieveOpenWaitWithRange	(cstring, cstring, uint32, uint16, int32, int32, cstring)
%
% if timeout_sec = 0 , it is retrieveOpenDirect() or retrieveOpen()

index_server = getenv('INDEXSERVERNAME');
liblabcom = 'libretrieve';
calllib( liblabcom, 'IndexNoClosing',1);
if strcmp(host,'')
    lib_func = 'retrieveOpenWaitWithRange';
    rtd = calllib( liblabcom, lib_func, diag_name, index_server, shot_number, sub_shot, wait_time_sec, range_type, range_str);
else
    lib_func = 'retrieveOpenDirectWait';
    rtd = calllib( liblabcom, lib_func, diag_name, host, path, shot_number, sub_shot, 0, wait_time_sec);
end
%
if rtd < 0 
    if verbose
        retrieveMessage(rtd, lib_func);
    end
    calllib( liblabcom, 'IndexNoClosing',0);
    return;
end

end
function code = mRetrieveClose( rtd, verbose)
% Close process for retrieve-Protocol 
%   code = mRetrieveClose( rtd, verbose)
%

%
% RETRIEVE_API int retrieveClose(int rtr_d);
%
liblabcom = 'libretrieve';
lib_func = 'retrieveClose';
code = calllib( liblabcom, lib_func, rtd);

if code < 0 
    if verbose
        retrieveMessage(code, lib_func);
    end
end
calllib( liblabcom, 'IndexNoClosing',0);
end

function [code, last_channel_number, syy, smm, sdd, shour, smin, ssec, trans_name] = ...
    mRetrieveShotInfo( rtd, verbose)
% Get Shot Information with opened retrieve-Protocol
%   [code last_channel_number syy smm sdd shour smin ssec trans_name] = ...
%       mRetrieveShotInfo( rtd, verbose)
%

%
%RETRIEVE_API int retrieveShotInfo(int rtr_d ,unsigned int* n_channel ,
%							   short* year ,short* month ,short* day ,short* hour ,short* min ,short* sec ,
%							   char *management ,char* comment ,int comment_size ,char *server ); 
pn_channel = libpointer('uint32Ptr',zeros(1,'uint32'));
pyear = libpointer('int16Ptr',zeros(1,'int16'));
pmonth = libpointer('int16Ptr',zeros(1,'int16'));
pday = libpointer('int16Ptr',zeros(1,'int16'));
phour = libpointer('int16Ptr',zeros(1,'int16'));
pmin = libpointer('int16Ptr',zeros(1,'int16'));
psec = libpointer('int16Ptr',zeros(1,'int16'));

comment_size = int32(128);

lib_func = 'retrieveShotInfo';
[code, last_channel_number, syy, smm, sdd, shour, smin, ssec, ~, ~, trans_name] = ...
    calllib( 'libretrieve', lib_func, rtd, pn_channel, pyear, pmonth, ...
    pday, phour, pmin, psec, blanks(128), blanks(128),comment_size, blanks(128)); 
if code < 0 
    if verbose
        retrieveMessage(code, lib_func);
    end
    mRetrieveClose( rtd, verbose);
    return;
end
end
function [code, data_bytes, comp_bytes, param_count, data_type, image_type, value_len, is_nframe, comment] = ...
    mRetrieveChInfo( rtd, ch_no, verbose)
% Get Channel Information with opened retrieve-Protocol
%   [code data_bytes comp_bytes param_count data_type image_type ...
%      value_len is_nframe comment] = mRetrieveChInfo( rtd, ch_no, verbose)
%

%
%RETRIEVE_API int retrieveChInfo( int rtr_d ,int ch_no ,
%			unsigned int* data_length ,unsigned int* comp_length ,unsigned short* param_count ,
%			short* data_type ,char* image_type ,unsigned short* value_len ,int* is_nframe ,
%			char* management ,char* comment ,int comment_size );

pdata_bytes = libpointer('int64Ptr',zeros(1,'int64'));
pcomp_bytes = libpointer('int64Ptr',zeros(1,'int64'));
pparam_count = libpointer('uint16Ptr',zeros(1,'uint16'));
pdata_type = libpointer('int16Ptr',zeros(1,'int64'));
pvalue_len = libpointer('uint16Ptr',zeros(1,'uint16'));
pis_nframe = libpointer('int32Ptr',zeros(1,'int32'));
comment_size = int32(128);

lib_func = 'retrieveChInfo_ex';
image_type = '';
[code, data_bytes, comp_bytes, param_count, data_type, image_tmp, value_len, is_nframe,~,comment] = ...
    calllib( 'libretrieve', lib_func, rtd, ch_no, pdata_bytes, pcomp_bytes, ...
    pparam_count, pdata_type, blanks(32), pvalue_len, pis_nframe, blanks(32), blanks(128), comment_size); 
if code < 0 
    if verbose
        retrieveMessage(code, lib_func);
    end
    mRetrieveClose( rtd, verbose);
    return;
end
% add 2012-08-22
image_type = strtrim(image_tmp);
end

function [code, data_bytes, comp_bytes, param_count, data_type, image_type, value_len, is_nframe, comment, ch_no] = ...
    mRetrieveChInfo2( rtd, signal_name, verbose)
% Get Channel Information with opened retrieve-Protocol and name of siganl
%   [code data_bytes comp_bytes param_count data_type image_type ...
%      value_len is_nframe comment ch_no] = mRetrieveChInfo2( rtd, signal_name, verbose)
%

%
%RETRIEVE_API int retrieveChInfo2( int rtr_d, char* signal_name,
%			unsigned int* data_length ,unsigned int* comp_length ,unsigned short* param_count ,
%			short* data_type ,char* image_type ,unsigned short* value_len ,int* is_nframe ,
%			char* management ,char* comment ,int comment_size, int* ch_no );

pdata_bytes = libpointer('int64Ptr',zeros(1,'int64'));
pcomp_bytes = libpointer('int64Ptr',zeros(1,'int64'));
pparam_count = libpointer('uint16Ptr',zeros(1,'uint16'));
pdata_type = libpointer('int16Ptr',zeros(1,'int16'));
pvalue_len = libpointer('uint16Ptr',zeros(1,'int16'));
pis_nframe = libpointer('int32Ptr',zeros(1,'int32'));
pch_no = libpointer('int32Ptr',zeros(1,'int32'));
comment_size = int32(128);

lib_func = 'retrieveChInfo2_ex';
image_type = '';
[code, ~, data_bytes, comp_bytes, param_count, data_type, image_tmp, value_len, is_nframe,~,comment, ch_no] = ...
    calllib( 'libretrieve', lib_func, rtd, signal_name, pdata_bytes, pcomp_bytes, ...
    pparam_count, pdata_type, blanks(32), pvalue_len, pis_nframe, blanks(32), blanks(128), comment_size, pch_no); 
if code < 0 
    if verbose
        retrieveMessage(code, lib_func);
    end
    mRetrieveClose( rtd, verbose);
    return;
end
% add 2012-08-22
image_type = strtrim(image_tmp);
end
function [code, data_ptr, data_length] = ...
    mRetrieveChData( rtd, ch_no, array_bytes, image_type, data_class, verbose)
% Get Channel block data with opened retrieve-Protocol
%   [code data_ptr data_length] = ...
%       mRetrieveChData( rtd, ch_no, array_bytes, image_type, verbose)
%

%
% RETRIEVE_API int retrieveChData( int rtr_d, int ch_no ,
%			void* data ,unsigned int array_size ,unsigned int* data_length );
%
data_ptr = libpointer; % NULL-pointer
data_length = int64(0);
code = 0;
if array_bytes < 1 
    data_length = int64(0);
    return;
end
if strcmp(data_class,'' )
    [data_class, data_byte] = mImageType2DataClass(image_type);
else
    data_byte = mClassName2bytesize( data_class );
end
array_size = array_bytes/data_byte;
pdata = libpointer([data_class 'Ptr'],zeros(array_size,1, data_class));
pdata_length = libpointer('int64Ptr',zeros(1,'int64'));

lib_func = 'retrieveChData_ex';
code = calllib( 'libretrieve', lib_func, rtd, ch_no, pdata, array_bytes,pdata_length);
if code < 0 
    if verbose
        retrieveMessage(code, lib_func);
    end
    mRetrieveClose( rtd, verbose);
    return;
end
data_length = pdata_length.Value/data_byte;
data_ptr = pdata;
end

function [code, params] = ...
    mRetrieveChParams( rtd, ch_no, param_num, verbose)
% Get Channel parameter with opened retrieve-Protocol
%   [code params] = ...
%       mRetrieveChParams( rtd, ch_no, param_num, verbose)
%

%
%RETRIEVE_API int retrieveChParam( int rtr_d, int ch_no , int param_no ,
%			char* param_name ,char* param_value ,int* param_type );
%

code = 0;
params(param_num,1) = struct('Pname',[],'Pvalue',[],'Ptype',[]);
pdata_type = libpointer('int32Ptr',zeros(1,'int32'));

lib_retrieve = 'libretrieve';
lib_func = 'retrieveChParam';
for param_idx = 0:param_num-1
    [code, param_name, param_value] = ...
        calllib( lib_retrieve, lib_func, rtd, ch_no, param_idx, blanks(64), blanks(128), pdata_type);
    if code < 0 
        if verbose
            retrieveMessage(code, lib_func);
        end
        mRetrieveClose( rtd, verbose);
        return;
    end
    param_no = param_idx+1;
    params(param_no).Pname = param_name;
    params(param_no).Pvalue = param_value;
    params(param_no).Ptype = pdata_type.Value;
end
end

function [code, data_ptr, data_length] = ...
    mRetrieveChV( rtd, ch_no, org_array_bytes, image_type, array_type, verbose)
% Get Channel block data with opened retrieve-Protocol, convert to volts.
%   [code data_ptr data_length] = ...
%      mRetrieveChV( rtd, ch_no, org_array_bytes, image_type, ...
%                    array_type, verbose)
%

data_ptr = libpointer; % NULL-pointer
data_length = int64(0);
code = 0;
if org_array_bytes < 1 
    return;
end
[~, data_byte] = mImageType2DataClass(image_type);
lib_func = 'retrieveChVsynthesizedNumber';
syn_num = calllib( 'libretrieve', lib_func, rtd, ch_no);
if syn_num < 1 
    if verbose
        retrieveMessage(-122, lib_func);
    end
    mRetrieveClose( rtd, verbose);
    return;
end

array_size = int64(org_array_bytes/syn_num/data_byte);
if array_type == 1 % float 
    pdata = libpointer('singlePtr',zeros(array_size,1,'single'));
else    
    pdata = libpointer('doublePtr',zeros(array_size,1,'double'));
end
pdata_length = libpointer('int64Ptr',zeros(1,'int64'));

lib_func = 'retrieveChV_ex';
%
%RETRIEVE_API int retrieveChV( int rtr_d, int ch_no ,
%			void* data ,unsigned int array_size ,unsigned int* data_length ,int array_type);

code = calllib( 'libretrieve', lib_func, rtd, ch_no, pdata, array_size, ...
    pdata_length, array_type); 
if code < 0 
    if verbose
        retrieveMessage(code, lib_func);
    end
    mRetrieveClose( rtd, verbose);
    return;
end
data_length = pdata_length.Value;
data_ptr = pdata;
end

function [code, data_ptr, data_length, frame_x, frame_y] = ...
    mRetrieveChFrameData( rtd, ch_no, frame_nums, verbose)
% Get frame data in channel with opened retrieve-Protocol.
%   [code data_ptr data_length frame_x frame_y] = ...
%       mRetrieveChFrameData( rtd, ch_no, frame_nums, verbose)
%           farmes_nums   : '1:100', '2' not '0'
%

data_length = 0;
data_ptr = libpointer; % NULL-pointer
frame_num = length( frame_nums );
data_cnt = 0;
for frame_no = frame_nums
    [code, array_bytes, image_type, frame_x, frame_y] = ...
        mRetrieveFrameInfo( rtd, ch_no, frame_no, verbose);
    if code < 0
        return;
    end
    [code, fdata_ptr, frame_length] = ...
        mRetrieveFrameData( rtd, ch_no, frame_no, frame_x, frame_y, image_type, verbose);
    if code < 0
        return;
    else
        if data_cnt == 0
            [data_class,data_bytes] = mImageType2DataClass(image_type);
            if frame_num == 1 && frame_length ~= (frame_x*frame_y)
                data_ptr = lib.pointer([data_class 'Ptr'],zeros(1,frame_length));
            else
                data_ptr = lib.pointer([data_class 'Ptr'],zeros(frame_x,frame_y,frame_num));
            end
        end
%        start_p = data_cnt*(array_bytes/byte_size) + 1;
%        end_p = start_p + (array_bytes/byte_size) - 1;
        start_p = data_cnt*(frame_length) + 1;
        end_p = start_p + (frame_length) - 1;
%       2020-05-20 I changed it because the substitution process is slow.
%        data_ptr.Value(start_p:end_p) = fdata_ptr.Value(1:frame_length);
        frame_memcopy(data_ptr,start_p,end_p,fdata_ptr,frame_length, data_bytes);
    end
    if data_cnt == 0
        init_frame_length = frame_length;
    else
        if frame_length ~= init_frame_length
            code = 1;
            return
        end
    end
    data_length = data_length + frame_length;
    data_cnt = data_cnt+1;
end
end


function [code, data_bytes, image_type, frame_x, frame_y] = ...
    mRetrieveFrameInfo( rtd, ch_no, frame_no, verbose)
% Get frame information in channel with opened retrieve-Protocol.
%   [code data_bytes image_type frame_x frame_y] = ...
%       mRetrieveFrameInfo( rtd, ch_no, frame_no, verbose)
%

%
% RETRIEVE_API int retrieveFrameInfo( int rtr_d ,int ch_no ,int frame_no ,
%			unsigned int* data_length ,char* image_type ,unsigned int* frame_x ,unsigned int* frame_y ); 
pdata_length = libpointer('uint32Ptr',zeros(1,'uint32'));
pframe_x = libpointer('uint32Ptr',zeros(1,'uint32'));
pframe_y = libpointer('uint32Ptr',zeros(1,'uint32'));

lib_func = 'retrieveFrameInfo';
[code, data_bytes, image_type, frame_x, frame_y] = calllib( 'libretrieve', lib_func, ...
    rtd, ch_no, frame_no, pdata_length, blanks(32), pframe_x, pframe_y);

if code < 0 
    if verbose
        retrieveMessage(code, lib_func);
    end
    mRetrieveClose( rtd, verbose);
    return;
end
end

function [code, data_ptr, data_length] = ...
    mRetrieveFrameData( rtd, ch_no, frame_no, frame_x, frame_y, image_type, verbose)
% Get frame data in channel with opened retrieve-Protocol.
%   [code data_ptr data_length] = ...
%       mRetrieveFrameData( rtd, ch_no, frame_no, frame_x, frame_y, ...
%                           image_type, verbose)
%

%
% RETRIEVE_API int retrieveFrameData( int rtr_d, int ch_no ,int frame_no ,
% char* data ,unsigned int array_size ,unsigned int* data_length );
%
data_ptr = libpointer; % NULL-pointer
data_length = 0;
code = 0;
if frame_x < 1 || frame_y < 1 
    data_length = 0;
    return;
end
[data_class, data_byte] = mImageType2DataClass(image_type);
array_bytes = frame_x * frame_y * data_byte;
pdata = libpointer([data_class 'Ptr'],zeros(frame_x, frame_y, data_class));
pdata_length = libpointer('uint32Ptr',zeros(1,'uint32'));

lib_func = 'retrieveFrameData';
code = calllib( 'libretrieve', lib_func, rtd, ch_no, frame_no, pdata, array_bytes,pdata_length);
if code < 0 
    if verbose
        retrieveMessage(code, lib_func);
    end
    mRetrieveClose( rtd, verbose);
    return;
end
data_length = pdata_length.Value/data_byte;
data_ptr = pdata;
end

function [code, begin_no, end_no] = ...
    mRetrieveRetrieveNo( rtd, ch_no)
% Get begin-no and end-no of retrieved data in channel with opened retrieve-Protocol.
%   [code begin_no end_no] = ...
%        mRetrieveRetrieveNo( rtd, ch_no)
%

%
% RETRIEVE_API int retrieveGetRetrieveNo( int rtr_d ,int ch_no , int64_t* begin_no, int64_t* end_no)
%
code = 0;
begin_no_ptr = libpointer('int64Ptr',zeros(1,'int64'));
end_no_ptr = libpointer('int64Ptr',zeros(1,'int64'));

lib_func = 'retrieveGetRetrieveNo';
[code, begin_no, end_no] = calllib( 'libretrieve', lib_func, rtd, ch_no, begin_no_ptr, end_no_ptr);
if code < 0 
    if false
        retrieveMessage(code, lib_func);
    end
    return;
end
end

function image_type = mValueLen2ImageType( value_len )
% Make image_type value_len
%   image_type = mValueLen2ImageType( value_len )
%
if value_len < 9
    image_type = 'INT8';
elseif value_len < 17
    image_type = 'INT16';
elseif value_len < 33
    image_type = 'INT32';
else
    image_type = 'INT64';
end
end
function [class_name, bytes] = mImageType2DataClass( image_type )
% Make class name image_type
%   [class_name bytes] = mImageType2DataClass( image_type )
%
switch image_type
    case 'INT8'
        bytes = 1;
        class_name='int8';
    case 'INT16'
        bytes = 2;
        class_name='int16';
    case 'INT32'
        bytes = 4;
        class_name='int32';
    case 'INT64'
        bytes = 8;
        class_name='int64';
    case 'FLT32'
        bytes = 4;
        class_name='single';
    case 'FLT64'
        bytes = 8;
        class_name='double';
    case 'YUV422'   % 2Byte
        bytes = 2;
        class_name='uint16';
    case 'YUY2'   % 2020-05-20
        bytes = 2;
        class_name='uint16';
    case 'BayerBG8' % 1Byte 12bit
        bytes = 1;
        class_name='uint8';
    case 'BayerBG16'    % 2Byte
        bytes = 2;
        class_name='uint16';
    otherwise
        value_len = 0;
        if strncmp(image_type,'GRAY',4 ) || strncmp(image_type,'GREY',4 )
            value_len = str2double(image_type(5:length(image_type)));
        else
            % 2020-05-20 strfind()->contains()
            if length(image_type) == 4 || length(image_type) == 3
                if contains(image_type,'R') && contains(image_type,'G') && ...
                        contains(image_type,'B')
                    value_len = 32;
                end
            end
        end
        if value_len < 9 
            bytes = 1;
            class_name = 'uint8';
        elseif value_len < 17 
            bytes = 2;
            class_name = 'uint16';
        elseif value_len < 33
            bytes = 4;
            class_name = 'uint32';
        else
            bytes = 8;
            class_name = 'int64';
        end
end
end
function bytesize = mClassName2bytesize( class_name )
% 
%   bytesize = mClassName2bytesize( class_name )
%
switch class_name
    case {'int8','uint8'}
        bytesize = 1;
    case {'int16','uint16'}
        bytesize = 2;
    case {'int32','uint32','single'}
        bytesize = 4;
    case {'int64','uint64','double'}
        bytesize = 8;
    otherwise
        bytesize = 1;
end
end
function [code, time_ptr, data_length, params] = ...
    mRetrieveChTime( rtd, diag_name, shot_number, sub_shot, ch_no, is_double, index_server, verbose)
% Get Channel time data with opened retrieve-Protocol
%   [code data_ptr data_length params] = ...
%       mRetrieveChTime( rtd, diag_name, shot_number, sub_shot, ch_no, is_double, index_server, verbose)
%

%
%RETRIEVE_API int retrieveTimeInfo( int rtr_d, int ch_no,
%					char* dts_source, char* dts_hostid, char*	dts_moduleid, char* dts_trig_ch,
%					char* clk_source, char* clk_hostid, char*	clk_moduleid, char* dts_clk_ch,
%					int* dts_user_define, char* clock_src, int64_t* time_array_size, 
%					void* clock_cycle, void* trigger_delay, void* sampling0timing, int is_double);
% [int32, cstring, cstring, cstring, cstring, cstring, cstring, cstring, cstring, int32Ptr, 
%  cstring, int64Ptr, voidPtr, voidPtr, voidPtr]  retrieveTimeInfo
% (int32, int32, cstring, cstring, cstring, cstring, cstring, cstring, cstring, cstring, 
%  int32Ptr, cstring, int64Ptr, voidPtr, voidPtr, voidPtr, int32)
%
%RETRIEVE_API int retrieveTimeData( int rtr_d, int ch_no,
%			void* data, int64_t array_size, int64_t* data_length,int is_double );
%[int32, voidPtr, int64Ptr]	retrieveTimeData	(int32, int32, voidPtr, int64, int64Ptr, int32)
%
code = 0;
time_ptr = libpointer; % NULL-pointer
data_length = 0;
if nargout > 3
   params(13,1) = struct('Pname',[],'Pvalue',[],'Ptype',[]);
end
liblabcom = 'libretrieve';

dts_user_ptr = libpointer('int32Ptr', zeros(1,'int32'));
time_arraysize_ptr = libpointer('int64Ptr', zeros(1,'int64'));
time_length_ptr = libpointer('int64Ptr', zeros(1,'int64'));

if is_double == 1
    trig_timing_ptr = libpointer('doublePtr',zeros(1,'double'));
    trig_delay_ptr = libpointer('doublePtr',zeros(1,'double'));
    clk_cycle_ptr = libpointer('doublePtr',zeros(1,'double'));
else
    trig_timing_ptr = libpointer('singlePtr',zeros(1,'single'));
    trig_delay_ptr = libpointer('singlePtr',zeros(1,'single'));
    clk_cycle_ptr = libpointer('singlePtr',zeros(1,'single'));
end

dts_source = blanks(64);
dts_host = blanks(64);
dts_mod = blanks(64);
clk_source = blanks(64);
clk_host = blanks(64);
clk_mod = blanks(64);
ext_or_int = blanks(64);
dts_trg_ch = blanks(64);
dts_clk_ch = blanks(64);

if verbose
    fprintf('---DTS Infomation---\n');
end
for n=1
    lib_func = 'retrieveTimeInfo';
    [code, dts_source, dts_host, dts_mod, dts_trg_ch, ...
           clk_source, clk_host, clk_mod, dts_clk_ch, ...
           ~, ext_or_int, ~, ~, ~, ~] = ...
    calllib( liblabcom, lib_func, rtd, ch_no, ...
           dts_source, dts_host, dts_mod, dts_trg_ch, ...
           clk_source, clk_host, clk_mod, dts_clk_ch, ...
           dts_user_ptr, ext_or_int, time_arraysize_ptr, ...
           clk_cycle_ptr, trig_delay_ptr, trig_timing_ptr,is_double);
%
% [int32, cstring, cstring, cstring, cstring, cstring, cstring, cstring, cstring, int32Ptr, 
%  cstring, int64Ptr, voidPtr, voidPtr, voidPtr]  retrieveTimeInfo
% (int32, int32, cstring, cstring, cstring, cstring, cstring, cstring, cstring, cstring, 
%  int32Ptr, cstring, int64Ptr, voidPtr, voidPtr, voidPtr, int32)
%       
    if code ~= 0
        if verbose 
            retrieveMessage( code, lib_func);
        end
        break;
    end
    if verbose
        fprintf('DTS source         = %s\n', dts_source);
        fprintf('DTS host ID        = %s\n', dts_host);
        fprintf('DTS module ID      = %s\n', dts_mod);
        fprintf('DTS trigger ch     = %s\n', dts_trg_ch);
        fprintf('CLK source         = %s\n', clk_source);
        fprintf('CLK host ID        = %s\n', clk_host);
        fprintf('CLK module ID      = %s\n', clk_mod);
        fprintf('DTS clock ch       = %s\n', dts_clk_ch);
        fprintf('Clock Source       = %s\n', ext_or_int);
        fprintf('DTS user define    = %10ld\n', dts_user_ptr.Value);
        fprintf('Time array size    = %10ld\n', time_arraysize_ptr.Value);
    end

    array_size = time_arraysize_ptr.Value;
    if is_double == 1
       time_array_ptr = libpointer('doublePtr',zeros(array_size,1,'double'));
    else
       time_array_ptr = libpointer('singlePtr',zeros(array_size,1,'single'));
    end

    lib_func = 'retrieveTimeData';
    code = calllib( liblabcom, lib_func, rtd, ch_no, ...
        time_array_ptr, time_arraysize_ptr.Value, time_length_ptr, is_double);
%RETRIEVE_API int retrieveTimeData( int rtr_d, int ch_no,
%			void* data, int64_t array_size, int64_t* data_length,int is_double );
%[int32, voidPtr, int64Ptr]	retrieveTimeData	
%        (int32, int32, voidPtr, int64, int64Ptr, int32)

    if code ~= 0 
        if verbose
            retrieve_tMessage( code, lib_func);
        end
        break;
    end
    if verbose 
        fprintf('Cycle   = [%2.8f] sec\n',clk_cycle_ptr.Value);
        fprintf('Delay   = [%2.8f] sec\n',trig_timing_ptr.Value);
    end
    time_ptr = time_array_ptr;
    data_length = array_size;
    
    if nargout > 3
%        params(13,1) = struct('Pname',[],'Pvalue',[],'Ptype',[]);

        params(1,1) = struct('Pname', 'DTSsource',    'Pvalue', dts_source, 'Ptype', 1);
        params(2,1) = struct('Pname', 'DTShostID',    'Pvalue', dts_host, 'Ptype', 1);
        params(3,1) = struct('Pname', 'DTSmoduleID',  'Pvalue', dts_mod, 'Ptype', 1);
        params(4,1) = struct('Pname', 'TriggerCh',    'Pvalue', dts_trg_ch, 'Ptype', 4);
        params(5,1) = struct('Pname', 'CLKsource',    'Pvalue', clk_source, 'Ptype', 1);
        params(6,1) = struct('Pname', 'CLKhostID',    'Pvalue', clk_host, 'Ptype', 1);
        params(7,1) = struct('Pname', 'CLKmoduleID',  'Pvalue', clk_mod, 'Ptype', 1);
        params(8,1) = struct('Pname', 'ClockCh',      'Pvalue', dts_clk_ch, 'Ptype', 4);
        params(9,1) = struct('Pname', 'ClockSource',  'Pvalue', ext_or_int, 'Ptype', 1);
        params(10,1) = struct('Pname', 'TimeArraySize','Pvalue', num2str(array_size), 'Ptype', 7);

        if is_double == 1
            params(11,1) = struct('Pname', 'ArrayDataType',   'Pvalue', 'Double', 'Ptype', 1);
            params(12,1) = struct('Pname', 'ClockCycle(sec)', 'Pvalue', num2str(clk_cycle_ptr.Value,15), 'Ptype', 6);
            params(13,1)= struct('Pname', 'StartTiming(sec)','Pvalue', num2str(trig_timing_ptr.Value,15), 'Ptype', 6);
        else
            params(11,1) = struct('Pname', 'ArrayDataType',   'Pvalue', 'Single', 'Ptype', 1);
            params(12,1) = struct('Pname', 'ClockCycle(sec)', 'Pvalue', num2str(clk_cycle_ptr.Value,7), 'Ptype', 5);
            params(13,1)= struct('Pname', 'StartTiming(sec)','Pvalue', num2str(trig_timing_ptr.Value,7), 'Ptype', 5);
        end
    end
    break;
end % end loop
if code ~= 0 
    mRetrieveClose( rtd, false);
end
end
