function [time_cell, params_cell, code] = ...
    retrieve_t(diag_name, shot_number, sub_shot, channel_number, varargin)
% Make time arrays for diagnostics's channels, or Retrieve DTS's parameter on SHOT.
%   [time_cell, params_cell, code] = retrieve_t( diag_name, ...
%               shot_number, sub_shot, channel_number, ...)
%
%       diag_name(text)     : name of diagnostics or DTS
%           DTS name's format are 'DTS:hostID','DTS:hostID:',
%                               'DTS:hostID:moduleID'
%           If it is diagnostics name, this function make time array,
%           otherwise this function retrieve DTS's parameter.
%       shot_number(numeric): SHOT number
%       sub_shot(numeric)   : SHOT sub number
%       channel_number(numeric, [vector],text)
%                           : channel number
%           range format is "n:m"(vector), single format is "n"(numeric), 
%            and both(e.g. [1,3,5:7]).
%           "0" is all channels.
%           (text)          : signal tag name (e.g. 'sig-tag1')
%
%
%       time_cell(cell)     : time data
%           If 'pointer' option , the type of time data is libpointer.
%       params_cell(cell)   : parameter data
%       code(numeric)       : result code
%           If code is 0, the result is success; otherwise is an error.
%
% OPTIONS :
%         'double'          : data is double precision.( default )
%         'single'          : data is single precision.
%         'float'           : same as 'single'
%         'silent'          : do not display messages.
%         'pointer'         : the type of time_array is libpointer.
%         'wait'            : wait for the appearance of data
%         'timeout',timeout_sec(numeric)
%                           : seconds of timeout for wait
%                             The default is 180 sec.
%
% Examples :
%   (a) Make time array for diagnostics's channel with DTS information on SHOT.
%
%   >> [time_cell,~,code] = retrieve_t('RADH',98000,1,2);
%   ---DTS Infomation---
%   Index Server Name  = dasindex.lhd.nifs.ac.jp
%   Shot Number        =      98000
%   Sub Shot Number    =          1
%   DTS source         = VME
%   DTS host ID        = 133.75.175.172
%   DTS module ID      = DMOD0
%   DTS trigger ch     =          5
%   DTS clock ch       =          2
%   DTS user define    =          1
%   Time array size    =    1000000
%   Cycle = [0.00001000] sec
%   Delay = [0.00000000] sec
%   >>
%
%   (b) Retrieve DTS's parameter on SHOT.
%
%   >>[~,param,code] = retrieve_t('DTS:VME01',98000,1,0);
%   ---DTS Infomation---
%   Index Server Name  = dasindex.lhd.nifs.ac.jp
%   Shot Number        =      98000
%   Sub Shot Number    =          1
%   DTS source         = VME01:DMOD0
%   Channel Range      = [   1  -    8 ]
%   >>printparam(param);
%                           Name                    Value     Type
%     1                     Shot                    98000    int32
%     1                  SubShot                        1    int32
%   ...........
%
% Last update : 2016-12-01
%
% See also
%     retrieve_td printParam getParam retrieve


ip = inputParser;
ip.KeepUnmatched = true;

ip.addRequired('diag_name', @ischar);
ip.addRequired('shot_number', @(x)isnumeric(x) && isscalar(x) && x > 0);
ip.addRequired('sub_shot',@(x)isnumeric(x) && isscalar(x) && x >= 0);
%ip.addRequired('channel_number',@(x)isnumeric(x) && all(x>=0));

%ip.parse( diag_name, shot_number, sub_shot, channel_number );
ip.parse( diag_name, shot_number, sub_shot );

wait_time_sec = 0;
flg_timeout = false;
verbose = true;
time_dbl = true;
nvarargin = length(varargin);
opt_pointer = false;
arg_check = true;
err_arg = '';

for idx = 1:nvarargin

    if isempty(varargin{idx})
        continue;
    elseif flg_timeout
        if isnumeric(varargin{idx}) && isscalar(varargin{idx})
            wait_time_sec = varargin{idx};
            flg_timeout = false;
            continue
        else
            arg_check = false;
            break
        end
    end
    
    if ~ischar(varargin{idx}) 
        err_arg = strcat(err_arg, [' arg#' num2str(idx+4)]);
        arg_check = false;
        break
    end
    switch varargin{idx}
        case 'double'
            time_dbl = true;
        case {'single','float'}
            time_dbl = false;
        case 'silent'
            verbose = false;
        case 'pointer'
            opt_pointer = true;
        case 'timeout'
            flg_timeout = true;
        case 'wait'
            if wait_time_sec == 0
                wait_time_sec = 180;
            end
        otherwise
            arg_check = false;
            err_arg = strcat(err_arg, [' ' varargin{idx}]);
    end
end
if nargout > 0
    time_cell = cell(0);
end
if nargout > 1
    params_cell = cell(0);
end
if not(ischar(channel_number) || ( isnumeric(channel_number) && all(channel_number>=0) ) )
    if verbose
        disp('<Error: illegal channles>');
        disp('Type "help retrieve_t".');
    end
    code = -2;
    return;
end

if( ~arg_check ) 
    if verbose
        disp(['<Error: illegal argument ' err_arg ' >']);
        disp('Type "help retrieve_t".');
    end
    code = -2;
    return;
end
liblabcom= 'libretrieve';
if ~loadLibLabcom( liblabcom, verbose )
    code = -9;  % System error
    return;
end
sp = strfind(diag_name,'DTS:');
if 1 == length(sp) && 1 == sp(1) 
    sp = strfind(diag_name,':');
    dts_name = diag_name(sp(1)+1:length(diag_name));
else
    dts_name = '';
end
calllib( liblabcom, 'IndexNoClosing',1);
if isempty(dts_name)
    % Make time array for diagnostics's channel with DTS information on SHOT.
    [time_cell, params_cell, code] = ...
        retrieve_time(diag_name, shot_number, sub_shot, channel_number, time_dbl, opt_pointer, wait_time_sec, verbose );
else
    % Retrieve DTS's parameter on SHOT.
    if ischar(channel_number) 
        if verbose
            disp('<Error: illegal channles>');
            disp('Type "help retrieve_t".');
        end
        code = -2;
        return;
    end
    [params_cell, code] = retrieve_dts(dts_name, shot_number, sub_shot, channel_number,  verbose );
end
calllib( liblabcom, 'IndexNoClosing',0);
end

function [time_cell, params_cell, code] = ...
    retrieve_time(diag_name, shot_number, sub_shot, channel_number, ...
    time_dbl, pointer, wait_time_sec, verbose )
% Make time array for diagnostics's channel with DTS information on SHOT.
%   [time_array_ptr, params_cell, code] = ...
%       retrieve_time( diag_name, shot_number, channel_number, ...
%                      time_dbl, pointer, wait_time_sec, verbose )
%
%       diag_name(text)     : diagnostics name
%       shot_number(numeric): SHOT number
%       sub_shot(numeric)   : SHOT sub number
%       channel_number(numeric)
%                           : channel number(single only)
%       time_dbl(logical)   : true is double precision array.
%                             false is single precision array. 
%       pointer(logical)    : true is .
%                             false is . 
%       wait_time_sec(numeric)
%                           :
%       verbose(logical)    : true is output messages to a display.
%                             false is donot output messages to a display. 
%
%       time_cell(cell)     : time data
%       params_cell(cell)   : parameter data
%       code(numeric)       : result code
%             if code is 0, the result is success; otherwise is an error.
%

if nargout > 0
    time_cell = cell(0);
end
if nargout > 1
    params_cell = cell(0);
end
liblabcom = 'libretrieve';
index_server = getenv('INDEXSERVERNAME');

[rtd, index_server] = ...
        mRetrieveOpen( diag_name, shot_number, sub_shot, '', '', ...
        wait_time_sec, verbose);
if rtd < 0
    code=rtd;
    return;
end
[code,last_channel_number,syy,smm,sdd,shour,smin,ssec,trans_server] = ...
mRetrieveShotInfo( rtd, verbose);
if code < 0
    return
end

if ischar(channel_number)
    type_ch = 0;
else
    type_ch = 2;
    if any(channel_number > last_channel_number )
        code = -1;
        if verbose
            retrieve_tMessage( code);
        end
        return;
    end
    if 0 == channel_number(1)
       channel_number = 1:last_channel_number;
    end
end
idx = 1;
dts_trig_ptr = libpointer('int16Ptr', zeros(1,'int16'));
dts_clk_ptr = libpointer('int16Ptr', zeros(1,'int16'));
dts_user_ptr = libpointer('int32Ptr', zeros(1,'int32'));
dts_arraysize_ptr = libpointer('int64Ptr', zeros(1,'int64'));

if time_dbl
    is_double = 1;
    trig_timing_ptr = libpointer('doublePtr',zeros(1,'double'));
    clk_cycle_ptr = libpointer('doublePtr',zeros(1,'double'));
else
    is_double = 0;
    trig_timing_ptr = libpointer('singlePtr',zeros(1,'single'));
    clk_cycle_ptr = libpointer('singlePtr',zeros(1,'single'));
end

pre_samp_ptr = libpointer('int32Ptr', zeros(1,'int32'));
samp_interval_ptr = libpointer('int32Ptr', zeros(1,'int32'));
clk_dts_id_ptr = libpointer('int32Ptr', zeros(1,'int32'));
real_ch_ptr = libpointer('int32Ptr',zeros(1,'int32'));
shot_no_ptr = libpointer('uint32Ptr',zeros(1,'uint32'));
sub_shot_ptr = libpointer('uint16Ptr',zeros(1,'uint16'));
last_count_ptr = libpointer('int64Ptr',zeros(1,'int64'));

if verbose
    fprintf('---DTS Infomation---\n');
    fprintf('Index Server Name  = %s\n', index_server);
    fprintf('Shot Number        = %10d\n', shot_number);
    fprintf('Sub Shot Number    = %10d\n', sub_shot);
end
if shot_number < 56221
    need_dtslink = 0;
else
    need_dtslink = 1;
end

for chno = channel_number
    
    dts_source= blanks(64);
    dts_host= blanks(64);
    dts_mod= blanks(64);
    clk_source= blanks(64);
    clk_host= blanks(64);
    clk_mod= blanks(64);
    dts_trg_ch = blanks(64);
    dts_clk_ch = blanks(64);
    ext_or_int= blanks(64);

    if shot_number < 56221 
        lib_func = 'retrieveGetDTSinfox';
		[code, ~, ~, dts_source, dts_host, dts_mod, ~, ~, ~, ~, ext_or_int,...
            ~, ~, ~, clk_dts_module] = ...
            calllib( liblabcom, lib_func,diag_name, shot_number, sub_shot, ...
                    chno, chno, index_server, blanks(64), blanks(64), blanks(64),...
					dts_trig_ptr, dts_clk_ptr, dts_user_ptr, dts_arraysize_ptr,...					
                    blanks(64), pre_samp_ptr, samp_interval_ptr, clk_dts_id_ptr, blanks(64));
         if code ~= 0
             break;
         end
         clk_source = dts_source;
         clk_host = dts_host;
         clk_mod = dts_mod;
         dts_trg_ch = num2str(dts_trig_ptr.value);
         dts_clk_ch = num2str(dts_clk_ptr.value);
    end
%    str = 'string variable';
%    vp = libpointer('voidPtr',[int8(str) 0]);
    if 0 == type_ch 
        ch_ptr = libpointer('voidPtr',[int8(channel_number) 0]);
    else
        ch_ptr = libpointer('int16Ptr',chno);
    end
    lib_func = 'retrieveGetDTSinfoFromRetrieve_ex';
    [code, ~, ~, ~, ~, dts_source, dts_host, dts_mod, dts_trg_ch, ...
        clk_source, clk_host, clk_mod, dts_clk_ch, ~, ~, ...
        ext_or_int, internal_clock, samp_interval, pre_samp, ~] = ...
           calllib( liblabcom, lib_func, rtd, type_ch, ch_ptr, need_dtslink,...
           real_ch_ptr, shot_no_ptr, sub_shot_ptr, ...
           dts_source, dts_host, dts_mod, dts_trg_ch, ...
           clk_source, clk_host, clk_mod, dts_clk_ch, ...
           dts_user_ptr, dts_arraysize_ptr, ...
           blanks(64), blanks(64), blanks(64),  blanks(64), last_count_ptr);
%[int32, voidPtr, int32Ptr, uint32Ptr, uint16Ptr, cstring, cstring, cstring, 
% cstring, cstring, cstring, cstring, cstring, int32Ptr, int32Ptr, cstring, 
% cstring, cstring, cstring, int64Ptr]	retrieveGetDTSinfoFromRetrieve	
%(int32, int32, voidPtr, int32, int32Ptr, uint32Ptr, uint16Ptr, 
% cstring, cstring, cstring, cstring, cstring, cstring, cstring, cstring, 
% int32Ptr, int32Ptr, cstring, cstring, cstring, cstring, int64Ptr)
%
    if code ~= 0
        if verbose
            retrieveMessage( code, lib_func);
        end
        break;
    end
    if verbose
        fprintf('\n');
        fprintf('DTS source         = %s\n', dts_source);
        fprintf('DTS host ID        = %s\n', dts_host);
        fprintf('DTS module ID      = %s\n', dts_mod);
        fprintf('DTS trigger ch     = %s\n', dts_trg_ch);
        fprintf('CLK source         = %s\n', clk_source);
        fprintf('CLK host ID        = %s\n', clk_host);
        fprintf('CLK module ID      = %s\n', clk_mod);
        fprintf('DTS clock ch       = %s\n', dts_clk_ch);
        fprintf('Clock Source       = %s\n', ext_or_int);
        fprintf('DTS user define    = %10d\n', dts_user_ptr.Value);
        fprintf('Time array size    = %10d\n', dts_arraysize_ptr.Value);
    end

    array_size = dts_arraysize_ptr.Value;
    if time_dbl
       time_array_ptr = libpointer('doublePtr',zeros(array_size,1,'double'));
    else
       time_array_ptr = libpointer('singlePtr',zeros(array_size,1,'single'));
    end

    lib_func = 'retrieveGetDTSdatax2_ex';
    code = calllib( liblabcom, lib_func, index_server, ...
        dts_source, dts_host, dts_mod, dts_trg_ch, ...
        clk_source, clk_host, clk_mod, dts_clk_ch, ...
        dts_user_ptr, array_size, shot_number, sub_shot, ...
        ext_or_int, internal_clock, samp_interval, pre_samp, last_count_ptr.value, ...
        1, is_double, ...
        time_array_ptr, clk_cycle_ptr, trig_timing_ptr);

    if code ~= 0 
        if verbose
            retrieve_tMessage( code, lib_func);
        end
        break;
    end
    if verbose 
        fprintf('Cycle   = [%2.8f] sec\n',clk_cycle_ptr.Value);
        fprintf('Delay   = [%2.8f] sec\n',trig_timing_ptr.Value);
    end
    if verbose
        fprintf('Channel = %d\n', real_ch_ptr.value);
    end
    if pointer
        time_cell{idx,1} = time_array_ptr;
    else
        time_cell{idx,1} = time_array_ptr.value;
    end
    if nargout > 1
        params(13,1) = struct('Pname',[],'Pvalue',[],'Ptype',[]);

        params(1,1) = struct('Pname', 'DTSsource',    'Pvalue', dts_source, 'Ptype', 1);
        params(2,1) = struct('Pname', 'DTShostID',    'Pvalue', dts_host, 'Ptype', 1);
        params(3,1) = struct('Pname', 'DTSmoduleID',  'Pvalue', dts_mod, 'Ptype', 1);
        params(4,1) = struct('Pname', 'TriggerCh',    'Pvalue', dts_trg_ch, 'Ptype', 4);
        params(5,1) = struct('Pname', 'CLKsource',    'Pvalue', clk_source, 'Ptype', 1);
        params(6,1) = struct('Pname', 'CLKhostID',    'Pvalue', clk_host, 'Ptype', 1);
        params(7,1) = struct('Pname', 'CLKmoduleID',  'Pvalue', clk_mod, 'Ptype', 1);
        params(8,1) = struct('Pname', 'ClockCh',      'Pvalue', dts_clk_ch, 'Ptype', 4);
        params(9,1) = struct('Pname', 'ClockSource',  'Pvalue', ext_or_int, 'Ptype', 1);
        params(10,1) = struct('Pname', 'TimeArraySize','Pvalue', num2str(array_size), 'Ptype', 7);

        if time_dbl
            params(11,1) = struct('Pname', 'ArrayDataType',   'Pvalue', 'Double', 'Ptype', 1);
            params(12,1) = struct('Pname', 'ClockCycle(sec)', 'Pvalue', num2str(clk_cycle_ptr.Value,15), 'Ptype', 6);
            params(13,1)= struct('Pname', 'StartTiming(sec)','Pvalue', num2str(trig_timing_ptr.Value,15), 'Ptype', 6);
        else
            params(11,1) = struct('Pname', 'ArrayDataType',   'Pvalue', 'Single', 'Ptype', 1);
            params(12,1) = struct('Pname', 'ClockCycle(sec)', 'Pvalue', num2str(clk_cycle_ptr.Value,7), 'Ptype', 5);
            params(13,1)= struct('Pname', 'StartTiming(sec)','Pvalue', num2str(trig_timing_ptr.Value,7), 'Ptype', 5);
        end
        params_cell{idx,1} = params;
    end
    if type_ch == 0
        break;
    end
    idx = idx + 1;
end
    mRetrieveClose(rtd);
    return
end

function [params_cell, code] = retrieve_dts(dts_name, shot_number, sub_shot, channel_number,  verbose )
% Retrieve DTS's parameter on SHOT.
%   [params_cell, code] = retrieve_dts( dts_name, ...
%                         shot_number, sub_shot, channel_number,  verbose )
%
%       dts_name(text)      : DTS name ('hostID','hostID:','hostID:moduleID' )
%       shot_number(numeric): SHOT number
%       sub_shot(numeric)   : SHOT sub number
%       channel_number(numeric)
%                           : channel number
%           range format is "n:m", single format is "n", 0 is all channels.
%       verbose(logical)    : true is output messages to a display.
%                             false is donot output messages to a display. 
%
%
%       params_cell(cell)   : DTS's parameter, 1-cell is 1-channel.
%       code(numeric)       : result code
%             if code is 0, the result is success; otherwise is an error.
%
params_cell = cell(0);
mod_name = '';
sp = strfind(dts_name,':');
if isempty(sp)
    host_name = dts_name;
else
    host_name = dts_name(1:sp(1)-1);
    if 1 == length(sp)
        mod_name = dts_name(sp(1)+1:length(dts_name));
    else
        mod_name = dts_name(sp(1)+1:sp(2)-1);
    end
end
if isempty(mod_name)
    mod_name = 'NONAME';
end
start_ch = min(channel_number);
end_ch = max(channel_number);
if 0 == start_ch
    start_ch = 1;
    end_ch = 0;
end
%
%RETRIEVE_API int retrieveGetDTSParametersCount(char *IndexServer,char *hostname,char *modname,
%			unsigned int shot,	unsigned short subshot,
%			short* param_count,	int* start_ch, int* end_ch, char *true_module_name);
liblabcom='libretrieve';
lib_func ='retrieveGetDTSParametersCount';
index_server = getenv('INDEXSERVERNAME');
pcnt_ptr = libpointer('int16Ptr',zeros(1,'int16'));
sch_ptr = libpointer('int32Ptr',start_ch);
ech_ptr = libpointer('int32Ptr',end_ch);
[code,~,~,~,~,~,~,x_mod_name] = ...
    calllib( liblabcom, lib_func, index_server, host_name, mod_name, ...
    shot_number, sub_shot, pcnt_ptr, sch_ptr,ech_ptr,blanks(64));
if code ~= 0 
    if verbose
        retrieve_tMessage(code, lib_func);
    end
    return;
end
if 0 == end_ch 
    channel_number = 1:ech_ptr.value;
end
if verbose
    fprintf('---DTS Infomation---\n');
	fprintf('Index Server Name  = %s\n', index_server);
	fprintf('Shot Number        = %10d\n', shot_number);
	fprintf('Sub Shot Number    = %10d\n', sub_shot);
    fprintf('DTS source         = %s:%s\n', host_name, x_mod_name);
	fprintf('Channel Range      = [ %3d  -  %3d ]\n', min(channel_number), max(channel_number));
end
%
%RETRIEVE_API int retrieveGetDTSParameters2(char *IndexServer,
%		char *hostname,	char *modname, unsigned int shot, unsigned short subshot,
%		unsigned short channel,	unsigned short paramterCount,
%		void *param_name, void *param_value, int *param_type);
%
%
%RETRIEVE_API void bufferStringCopy(char* string,void* buffer,int element_size,int number);
%
params_cell = cell(length(channel_number),1);
param_cnt = pcnt_ptr.Value;
pname_ptr = libpointer('voidPtr',zeros(param_cnt, 64,'uint8'));
pval_ptr =  libpointer('voidPtr',zeros(param_cnt, 128,'uint8'));
ptype_ptr = libpointer('int32Ptr',zeros(param_cnt, 1,'int32'));
params(param_cnt,1) = struct('Pname',[],'Pvalue',[],'Ptype',[]);
idx = 1;
lib_func = 'retrieveGetDTSParameters2';
for chno = channel_number
    [code] = ...
        calllib( liblabcom, lib_func, index_server, host_name, mod_name, ...
        shot_number, sub_shot, chno, param_cnt, pname_ptr, pval_ptr, ptype_ptr);
    if code ~= 0 
        if verbose
            retrieve_tMessage( code,lib_func);
        end
        return;
    end
    for pno = 1:param_cnt
        params(pno).Pname = calllib('libretrieve', 'bufferStringCopy', blanks(64), pname_ptr, 64, pno);
        params(pno).Pvalue = calllib('libretrieve', 'bufferStringCopy', blanks(128), pval_ptr, 128, pno);
        params(pno).Ptype = ptype_ptr.Value(pno);
    end
    params_cell{idx,1}=params;
    idx = idx+1;
end

end


function [rtd, index_server] = ...
    mRetrieveOpen( diag_name, shot_number, sub_shot, host, path, wait_time_sec, verbose)
% Open process for retrieve-Protocol
%   [rtd, index_server] = ...
%        mRetrieveOpen( diag_name, shot_number, sub_shot, host, path, ...
%        wait_time_sec, verbose)
%
% 

%
%RETRIEVE_API int retrieveOpenWait( const char *diag_name ,const char* index_server ,
%							   unsigned int shot_number, unsigned short sub_shot ,int timeout_sec );
%       if timeout_sec = 0 is retrieveOpen().
%
%RETRIEVE_API int retrieveOpenDirectWait( const char *diag_name ,const char* server ,const char* path,
%							   unsigned int shot_number, unsigned short sub_shot ,int port ,int timeout_sec );
%
% if timeout_sec = 0 , it is retrieveOpenDirect() or retrieveOpen()

index_server = getenv('INDEXSERVERNAME');
if strcmp(host,'')
    lib_func = 'retrieveOpenWait';
    rtd = calllib( 'libretrieve', lib_func, diag_name, index_server, shot_number, sub_shot, wait_time_sec);
else
    lib_func = 'retrieveOpenDirectWait';
    rtd = calllib( 'libretrieve', lib_func, diag_name, host, path, shot_number, sub_shot, 0, wait_time_sec);
end
%
if rtd < 0 
    if verbose
        retrieveMessage(rtd, lib_func);
    end
    return;
end
end
function code = mRetrieveClose( rtd, verbose)
% Close process for retrieve-Protocol 
%   code = mRetrieveClose( rtd, verbose)
%

%
% RETRIEVE_API int retrieveClose(int rtr_d);
%
lib_func = 'retrieveClose';
code = calllib( 'libretrieve', lib_func, rtd);

if code < 0 
    if verbose
        retrieveMessage(code, lib_func);
    end
end
end

function [code, last_channel_number, syy, smm, sdd, shour, smin, ssec, trans_name] = ...
    mRetrieveShotInfo( rtd, verbose)
% Get Shot Information with opened retrieve-Protocol
%   [code last_channel_number syy smm sdd shour smin ssec trans_name] = ...
%       mRetrieveShotInfo( rtd, verbose)
%

%
%RETRIEVE_API int retrieveShotInfo(int rtr_d ,unsigned int* n_channel ,
%							   short* year ,short* month ,short* day ,short* hour ,short* min ,short* sec ,
%							   char *management ,char* comment ,int comment_size ,char *server ); 
pn_channel = libpointer('uint32Ptr',zeros(1,'uint32'));
pyear = libpointer('int16Ptr',zeros(1,'int16'));
pmonth = libpointer('int16Ptr',zeros(1,'int16'));
pday = libpointer('int16Ptr',zeros(1,'int16'));
phour = libpointer('int16Ptr',zeros(1,'int16'));
pmin = libpointer('int16Ptr',zeros(1,'int16'));
psec = libpointer('int16Ptr',zeros(1,'int16'));

comment_size = int32(128);

lib_func = 'retrieveShotInfo';
[code, last_channel_number, syy, smm, sdd, shour, smin, ssec, ~, ~, trans_name] = ...
    calllib( 'libretrieve', lib_func, rtd, pn_channel, pyear, pmonth, ...
    pday, phour, pmin, psec, blanks(128), blanks(128),comment_size, blanks(128)); 
if code < 0 
    if verbose
        retrieveMessage(code, lib_func);
    end
    mRetrieveClose( rtd, verbose);
    return;
end
end

