function [time_array, params_cell, code] = ...
    retrieve_td( shot_number, sub_shot, dts_source, dts_host, dts_mod, ...
    trig_ch, clk_ch, ext_or_int, define_clk , array_size, varargin )
% Make time array with DTS information on SHOT.
%   [time_array, params_cell, code] = ...
%       retrieve_td( shot_number, sub_shot, dts_source, dts_host, dts_mod, ...
%       trig_ch, clk_ch, ext_or_int, define_clk , array_size, ... )
%
%       shot_number(numeric): SHOT number
%       sub_shot(numeric)   : SHOT sub number
%       dts_source(text)    : DTS source type ('VME','CAMAC',....)
%       dts_host(text)      : DTS Host ID ('VME01','133.75.175.172',...)
%       dts_mod(text)       : DTS module ID ('DMOD0','DMOD1',DMODSS0')
%       trig_ch(numeric)    : trigger channel number
%       clk_ch(numeric)     : clock channel number            ('External')
%       ext_or_int(text)    : 'External' or 'Internal'
%           If it is 'External', clock used DTS. 
%           If it is 'Internal', clock used 'define_clk'. 
%       define_clk(numeric) : clock speed(nano-seconds)       ('Internal')
%       array_size(numeric) : array size
%
%       time_array(array,libpointer)
%                           : time data
%           If 'pointer' option , the type of time_array is libpointer.
%       params_cell(cell)   : parameter data
%       code(numeric)       : result code
%           If code is 0, the result is success; otherwise is an error.
%
% OPTIONS :
%         'double'          : time_array is double precision.(default)
%         'single'          : time_array is single precision.
%         'float'           : same as 'single'
%         'silent'          : do not display messages.
%         'pointer'         : the type of time_array is libpointer.
%         'clksource',clkSource(text)
%                           : If CLK differs from DTS, Clock DTS Source (''VME'', ''FPGA'' etc..)
%         'clkhost',clkHostID(text)
%                           : If CLK differs from DTS, Clock Host ID (''133.75.175.180'' etc..)'
%         'clkmodule',clkModuleID(text)
%		                    : If CLK differs from DTS, Clock Module ID (''DMODSS0'', ''DMOD0'' etc..)'
%
% Examples  :
%   (a) When 'clk_source' is 'External',
%   >> data = retrieve('RADH',98000,1,1);
%    .....
%   >> tm = retrieve_td(98000,1,'VME','133.75.175.172','DMOD0', ...
%        5,2,'External',0,length(data{1}));
%    ....
%   >> plot( tm, data{1});
%         
%   (b) When 'clk_source' is 'Internal',
%   >> [time_buff, param, code] = retrieve_td( 98000,1, ...
%                   'VME','VME01','DMOD0',2,0,'Internal',100000,10000);
%         
% Last update : 2016-12-01
%
% See also
%     retrieve_t printParam getParam 
if nargout > 0
   time_array = 0;
end
if nargout > 1
   params_cell= cell(0);
end
if nargout > 2
    code = -1;
end
ip = inputParser;
ip.KeepUnmatched = true;

ip.addRequired('shot_number', @(x)isnumeric(x) && isscalar(x) && x > 0);
ip.addRequired('sub_shot',@(x)isnumeric(x) && isscalar(x) && x >= 0);
ip.addRequired('dts_source', @ischar);
ip.addRequired('dts_host', @ischar);
ip.addRequired('dts_mod', @ischar);
ip.addRequired('trig_ch', @(x)isnumeric(x) && isscalar(x));
ip.addRequired('clk_ch', @(x)isnumeric(x) && isscalar(x));
ip.addRequired('ext_or_int', @ischar);
ip.addRequired('define_clk',@(x)isnumeric(x) && isscalar(x) && x >= 0);
ip.addRequired('array_size',@(x)isnumeric(x) && isscalar(x) && x >= 0);

ip.parse( shot_number, sub_shot, dts_source, dts_host, dts_mod, ...
    trig_ch, clk_ch, ext_or_int, define_clk , array_size);

verbose = true;
is_double = 1;
nvarargin = length(varargin);
opt_pointer = false;
arg_check = true;
err_arg = '';

clk_source = '';
flg_clk_s = false;
clk_host = '';
flg_clk_h = false;
clk_mod = '';
flg_clk_m = false;
internal_clock = strcat(num2str(define_clk),'ns');
flg_intclk = false;
samp_interval = '1';
flg_sampinterval = false;
pre_samp = '0';
flg_presamp = false;
last_count = 0;
flg_lastcount = false;
is_set_clk = false;

for idx = 1:nvarargin
    if isempty(varargin{idx})
        continue;
    elseif flg_lastcount
        if isnumeric(varargin{idx}) && isscalar(varargin{idx})
            last_count = varargin{idx};
            flg_lastcount = false;
            continue
        else
            arg_check = false;
            break
        end
    end
    
    if ~ischar(varargin{idx}) 
        arg_check = false;
        break
    end
    if flg_clk_s
        clk_source = varargin{idx};
        flg_clk_s = false;
        is_set_clk = true;
        continue
    end
    if flg_clk_h
        clk_host = varargin{idx};
        flg_clk_h = false;
        is_set_clk = true;
        continue
    end
    if flg_clk_m
        clk_mod = varargin{idx};
        flg_clk_m = false;
        is_set_clk = true;
        continue
    end
    if flg_intclk
        internal_clock = varargin{idx};
        flg_intclk = false;
        continue
    end
    if flg_sampinterval
        samp_interval = varargin{idx};
        flg_sampinterval = false;
        continue
    end
    if flg_presamp
        pre_samp = varargin{idx};
        flg_presamp = false;
        continue
    end
    switch varargin{idx}
        case 'double'
            is_double = 1;
        case {'single','float'}
            is_double = 0;
        case 'silent'
            verbose = false;
        case 'pointer'
            opt_pointer = true;
        case 'clksource'
            flg_clk_s = true;
        case 'clkhost'
            flg_clk_h = true;
        case 'clkmodule'
            flg_clk_m = true;
        otherwise
            arg_check = false;
            err_arg = strcat(err_arg, [' ' varargin{idx}]);
    end
end
time_array=0;
if( ~arg_check ) 
    if verbose
        disp(['<Error: illegal argument ' err_arg ' >']);
        disp('Type "help retrieve_td".');
    end
    code = -2;
    return;
end

liblabcom = 'libretrieve';
if ~loadLibLabcom( liblabcom, verbose )
    code = -9;  % System error
    return;
end

if opt_pointer
    time_array = libpointer;
else
    time_array = 0;
end

index_server = getenv('INDEXSERVERNAME');
dts_trg_ch = num2str(trig_ch);
dts_clk_ch = num2str(clk_ch);
dts_user_ptr = libpointer('int32Ptr', define_clk);
dts_arraysize_ptr = libpointer('int64Ptr', array_size);

if ~flg_clk_s
    clk_source = dts_source;
end
if ~flg_clk_h
    clk_host = dts_host;
end
if ~flg_clk_m
    clk_mod = dts_mod;
end
if verbose
	fprintf('---DTS Infomation---\n');
	fprintf('Index Server Name  = %s\n', index_server);
	fprintf('Shot Number        = %10d\n', shot_number);
	fprintf('Sub Shot Number    = %10d\n', sub_shot);
	fprintf('DTS source         = %s\n', dts_source);
	fprintf('DTS host ID        = %s\n', dts_host);
	fprintf('DTS module ID      = %s\n', dts_mod);
	fprintf('DTS trigger ch     = %s\n', dts_trg_ch);
    if is_set_clk 
	fprintf('CLK source         = %s\n', clk_source);
	fprintf('CLK host ID        = %s\n', clk_host);
	fprintf('CLK module ID      = %s\n', clk_mod);
    end
	fprintf('DTS clock ch       = %s\n', dts_clk_ch);
	fprintf('DTS user define    = %10d\n', dts_user_ptr.Value);
	fprintf('Time array size    = %10d\n', dts_arraysize_ptr.Value);
end

if is_double == 1
    time_array_ptr = libpointer('doublePtr',zeros(1,array_size,'double'));
    trig_timing_ptr = libpointer('doublePtr',zeros(1,'double'));
    clk_cycle_ptr = libpointer('doublePtr',zeros(1,'double'));
else
    time_array_ptr = libpointer('singlePtr',zeros(1,array_size,'single'));
    trig_timing_ptr = libpointer('singlePtr',zeros(1,'single'));
    clk_cycle_ptr = libpointer('singlePtr',zeros(1,'single'));
end

lib_func = 'retrieveGetDTSdatax2_ex';
code = calllib( liblabcom, lib_func, index_server, ...
        dts_source, dts_host, dts_mod, dts_trg_ch, ...
        clk_source, clk_host, clk_mod, dts_clk_ch, ...
        dts_user_ptr, array_size, shot_number, sub_shot, ...
        ext_or_int, internal_clock, samp_interval, pre_samp, last_count, ...
        1, is_double, ...
        time_array_ptr, clk_cycle_ptr, trig_timing_ptr);

if code ~= 0 
    if verbose
        retrieve_tMessage(code, lib_func);
    end
    return;
end
if opt_pointer
    time_array = time_array_ptr;
else
    time_array = time_array_ptr.Value;
end

if verbose 
    fprintf('Cycle = [%2.8f] sec\n',clk_cycle_ptr.Value);
    fprintf('Delay = [%2.8f] sec\n',trig_timing_ptr.Value);
end
if nargout > 1
    if is_set_clk
        pnum = 13;
    else
        pnum = 10;
    end
    params(pnum) = struct('Pname',[],'Pvalue',[],'Ptype',[]);
    
    params(1) = struct('Pname', 'DTSsource',    'Pvalue', dts_source, 'Ptype', 1);
    params(2) = struct('Pname', 'DTShostID',    'Pvalue', dts_host, 'Ptype', 1);
    params(3) = struct('Pname', 'DTSmoduleID',  'Pvalue', dts_mod, 'Ptype', 1);
    params(4) = struct('Pname', 'TriggerCh',    'Pvalue', dts_trg_ch, 'Ptype', 4);
    if is_set_clk
    params(5) = struct('Pname', 'CLKsource',    'Pvalue', clk_source, 'Ptype', 1);
    params(6) = struct('Pname', 'CLKhostID',    'Pvalue', clk_host, 'Ptype', 1);
    params(7) = struct('Pname', 'CLKmoduleID',  'Pvalue', clk_mod, 'Ptype', 1);
    end
    params(pnum-5) = struct('Pname', 'ClockCh',      'Pvalue', dts_clk_ch, 'Ptype', 4);
    params(pnum-4) = struct('Pname', 'ClockSource',  'Pvalue', ext_or_int, 'Ptype', 1);
    params(pnum-3) = struct('Pname', 'TimeArraySize','Pvalue', num2str(array_size), 'Ptype', 7);

    if is_double == 1
        params(pnum-2) = struct('Pname', 'ArrayDataType',   'Pvalue', 'Double', 'Ptype', 1);
        params(pnum-1) = struct('Pname', 'ClockCycle(sec)', 'Pvalue', num2str(clk_cycle_ptr.Value,15), 'Ptype', 6);
        params(pnum)= struct('Pname', 'StartTiming(sec)','Pvalue', num2str(trig_timing_ptr.Value,15), 'Ptype', 6);
    else
        params(pnum-2) = struct('Pname', 'ArrayDataType',   'Pvalue', 'Single', 'Ptype', 1);
        params(pnum-1) = struct('Pname', 'ClockCycle(sec)', 'Pvalue', num2str(clk_cycle_ptr.Value,7), 'Ptype', 5);
        params(pnum)= struct('Pname', 'StartTiming(sec)','Pvalue', num2str(trig_timing_ptr.Value,7), 'Ptype', 5);
    end
    params_cell{1} = params;
end
end





