function tv3Monitor( diag_name, channel_number, varargin)
% Monitor image channel with RTTRansd2(transfer protocol ver.3).
%   tvMonitor( diag_name, channel_number[, control_port], varargin) 
%
%       diag_name(text)         : name of diagnostics
%       channel_number(numeric) : channel number
%       control_port(numeric)   : control port number
%
% OPTIONS :
%         'thinning',thinnings(numeric)
%                           : transfer thinning rate to 1/N (default: 1)
%         'udp' ,logical    : availability of UDP.  (default: false)
%         'host',hostname(text)
%                           : monitor from a direct server
%
% Example :
%
%   >> tv3Monitor('TSCCD',1);
%
% Last update : 2020-05-20
%
% See also
%     wave3Monitor
%

if nargin < 2
    disp('<Error: the argument is short.>');
    disp('Type "help tv3Monitor".');
    return;
end

ip = inputParser;

validScalarPosNum = @(x) isnumeric(x) && isscalar(x) && (x > 0);
addRequired(ip,'diag_name', @ischar);
addRequired(ip,'channel_number',validScalarPosNum);
addOptional(ip,'control_port',-1,validScalarPosNum);

addParameter(ip,'udp', 0, @(x)islogical(x));
addParameter(ip,'thinning', 1, @(x)isnumeric(x) && x > 0);
addParameter(ip,'host', '', @ischar);

parse(ip,diag_name,channel_number,varargin{:});

ctrl_port = ip.Results.control_port;
thinnings = ip.Results.thinning;
udp_use   = ip.Results.udp;
host_name = ip.Results.host;
verbose = true;

if udp_use
    packet_size = 0;
else
    packet_size = -1;
end
channel_list= num2str(channel_number);

while true
    code = RTCstart( verbose );
    if code ~= 0 
        break;
    end
    thin_type = 0;
	timeout = 500;
    desc = RTCopen(diag_name, host_name, channel_list, ctrl_port, packet_size, thinnings, thin_type, verbose);
    if desc < 0 
        break;
    end

    while true
        
        [status, pixel_dipth, ~, fr_width, fr_height,~,buf_type ] = ...
            RTCgetChannelParam( desc, channel_number, verbose );        
        if status < 0 
            break;
        end
        
        [code, image_type ] = ...
            RTCgetChParam( desc, channel_number, 'ImageType', verbose );
        if code ~= 0 
            break;
        end
        if strcmp(image_type,'IllegalParameterName' )
            image_type = 'unkown';
        end
        img=1; % RGB
        if strncmp(image_type,'GRAY', 4 ) || strncmp(image_type,'GREY', 4 )
            img=0; % GRAY
        elseif strcmp(image_type,'YUV422')
            img=2;
        elseif strcmp(image_type,'YUY2') 
            img=3; 
        end
        % TODO
        fprintf('dipth=%d,width=%d,height=%d,buf_type=%s,image=%s\n', ...
            pixel_dipth,fr_width,fr_height,buf_type,image_type);
        
        hf=figure;
        % figure(hf);        
        if 0 == img
            colormap(gray(double(2^pixel_dipth)));
        end
        stat_ptr = libpointer('int32Ptr',zeros(1,'int32'));
        shot_ptr = libpointer('int32Ptr',zeros(1,'int32'));
        sub_ptr = libpointer('int32Ptr',zeros(1,'int32'));
        blk_ptr = libpointer('int32Ptr',zeros(1,'int32'));
        head_ptr = libpointer('int64Ptr',zeros(1,'int64'));
        tail_ptr = libpointer('int64Ptr',zeros(1,'int64'));
        buf_ptr = libpointer([buf_type 'Ptr'],zeros(fr_width,fr_height));
        len_ptr = libpointer('int32Ptr',zeros(1,'int32'));

        code = RTCtransfer( desc, 1, verbose );
        if code ~= 0 
            break;
        end
        disp('Terminate when "q" or "Q" is entered.');
        while 1
            code = RTCgetChannelData(desc, channel_number, timeout, ...
            stat_ptr, shot_ptr, sub_ptr, ...
            blk_ptr, head_ptr, tail_ptr, buf_ptr, len_ptr, verbose );
            if code == 0 
                fprintf('blk=%d %d\n',blk_ptr.Value',stat_ptr.Value);
                if stat_ptr.Value == 8          % Acquire status
                    % figure(hf);
                    % if 17 > pixel_dipth 2020-05-20
                    if 0 == img
                        imshow(buf_ptr.Value');
                    elseif(2 == img || 3 == img )
                        imshow(yuv2rgb(buf_ptr,fr_width,fr_height,img-2),'Border','tight');
                    else
                        imshow(bgrx2rgb(buf_ptr,fr_width,fr_height));
                    end
                    title([diag_name ' CH#' num2str(channel_number)]);
                elseif stat_ptr.Value == 3      % Setup status
                    [code, pixel_dipth, ~, tmp_width, tmp_height,~,buf_type ] = ...
                    RTCgetChannelParam( desc, channel_number, verbose );        
                    if code < 0 
                        break;
                    end
                fprintf('tmp=%d %d\n',tmp_width ,tmp_height);
                    if tmp_width ~= fr_width | tmp_height ~= fr_height
                        fr_width = tmp_width;
                        fr_height = tmp_height;
                        buf_ptr = libpointer([buf_type 'Ptr'],zeros(fr_width,fr_height));
                    end
                else
                    pause(0.4);
                end
            elseif code == 1  % 1 is timeout
                pause(0.1);
            elseif code == 2  %
                break;
            elseif code < 0
                break;
            end
            if strcmpi(get(hf,'currentcharacter'),'q')
                break;
            end
        end
        close(hf);
        break;
    end
    RTCclose(desc, verbose);
    break;
   
end
RTCend(verbose);
if verbose
    disp('End.');
end
end

function code = RTCgetChannelData(desc, chno, timeout, stat_ptr, shot_ptr, sub_ptr, ...
    blk_ptr, head_ptr, tail_ptr, buf_ptr, len_ptr, verbose )
%
%
%
% loadLibLabcom() is called in RTCstart.
%
% stat_ptr = libpointer('int32Ptr',zeros(1,'int32'));
% shot_ptr = libpointer('int32Ptr',zeros(1,'int32'));
% sub_ptr = libpointer('int32Ptr',zeros(1,'int32'));
% blk_ptr = libpointer('int32Ptr',zeros(1,'int32'));
% head_ptr = libpointer('int64Ptr',zeros(1,'int64'));
% tail_ptr = libpointer('int64Ptr',zeros(1,'int64'));
% buf_ptr = libpointer('voidPtr',....);
% len_ptr = libpointer('int32Ptr',zeros(1,'int32'));

%
% int RTCgetChannelData( int desc, int chno, int timeout,
%                                 int *status, int *shotno, int *subshot,
%								 int *blkno, int64_t *headno, int64_t *tailno, char* buf, int *len )
lib_func = 'RTCgetChannelData';
code = calllib( 'librttrans', lib_func, desc, chno, timeout, stat_ptr, ...
    shot_ptr, sub_ptr, blk_ptr, head_ptr, tail_ptr, buf_ptr, len_ptr );
if code == 14
    return;
elseif code < 0
    if verbose
        rtc_message( code, lib_func);
    end
end
end

function [status, pixel_dipth, fr_rate, fr_width, fr_height, rec_len, buf_type] = ...
    RTCgetChannelParam( desc, chno, verbose )
%
%
%
% loadLibLabcom() is called in RTCstart.


% DllClass int RTCgetCamParam2( int desc, int chno, 
%			int* dipth, int* rate, int* width, int* height, int* reclen);

dipth_ptr = libpointer('int32Ptr',zeros(1,'int32'));
rate_ptr = libpointer('int32Ptr',zeros(1,'int32'));
width_ptr = libpointer('int32Ptr',zeros(1,'int32'));
height_ptr = libpointer('int32Ptr',zeros(1,'int32'));
rec_l_ptr = libpointer('int32Ptr',zeros(1,'int32'));
lib_func = 'RTCgetCamParam2';
code=calllib( 'librttrans', lib_func, desc, chno, ...
    dipth_ptr, rate_ptr, width_ptr, height_ptr, rec_l_ptr);

if code < 0
    if verbose
        rtc_message(code, lib_func);
    end
end

status = code;
pixel_dipth = dipth_ptr.Value;
fr_rate = rate_ptr.Value;
fr_width = width_ptr.Value;
fr_height = height_ptr.Value;
rec_len = rec_l_ptr.Value;

if pixel_dipth < 9
   buf_type = 'uint8';
elseif pixel_dipth < 17
   buf_type = 'uint16';
else
   buf_type = 'uint32';
end

end


function dst = bgrx2rgb(src_ptr, w, h)
%
%
%

dst_ptr = libpointer('uint8Ptr',zeros(h,w,3,'uint8'));
calllib( 'librttrans', 'LRTbgrx2rgb4matlab', dst_ptr, src_ptr, w,h);
dst = reshape(dst_ptr.Value,h,w,3);

end

function dst = yuv2rgb(src_ptr, w, h, ytype)
%
%
%

dst_ptr = libpointer('uint8Ptr',zeros(h,w,3,'uint8'));
calllib( 'librttrans', 'LRTyuv2rgb4matlab', dst_ptr, src_ptr, w,h, ytype);
dst = reshape(dst_ptr.Value,h,w,3);

end


function [code, pvalue] = RTCgetChParam(desc, chno, pkeyword, verbose )
%
%
%
% loadLibLabcom() is called in RTCstart.
%

%
% RTCgetChannelKeyParam( int desc, int chno, const char*key, char* buffer, int buffer_size);
lib_func = 'RTCgetChannelKeyParam';
[code, ~, pvalue] = calllib( 'librttrans', lib_func, desc, chno, pkeyword, blanks(128), 128 );
if code ~= 0
    if verbose
        rtc_message( code, lib_func);
    end
end
end

function code = RTCtransfer( desc, action, verbose )
%
%
%
% loadLibLabcom() is called in RTCstart.

%
% DllClass int RTCtransfer( int desc, int action);
lib_func = 'RTCtransfer';
code = calllib( 'librttrans', lib_func, desc, action);
if code ~= 0
    if verbose
        rtc_message(code, lib_func);
    end
end
end

function desc = RTCopen( diag_name, server, channel_list, control_port, ...
                                   packet_size, thinnings, thin_type, verbose )
%
%
%
% loadLibLabcom() is called in RTCstart.
%

% TODO comment out if external func.
%if ~exist('verbose','var')
%    verbose = true;
%end
%
% DllClass int RTCopen( char* diag, char* server, char* channel_list, 
%					   int control_port, int packet_size, int thinning, int thin_type );
lib_func = 'RTCopen';
desc = calllib( 'librttrans', lib_func, diag_name, server, channel_list, ...
        control_port, packet_size, thinnings, thin_type);
if desc < 0
    if verbose
        rtc_message(desc, lib_func);
    end
end
end

function code = RTCclose( desc, verbose )
%
%
%
% loadLibLabcom() is called in RTCstart.

%
% DllClass int RTCclose( int desc);
lib_func = 'RTCclose';
% [code]
code = calllib( 'librttrans', lib_func, desc);
if code ~= 0
    if verbose
        rtc_message(code, lib_func);
    end
end
end


function code = RTCstart( verbose )
%
%
%

liblabcom = 'librttrans';
if ~loadLibLabcom( liblabcom, verbose )
    code = -9;
    return;
end
%
% DllClass int RTCStart( void )
lib_func = 'RTCStart';
code = calllib( liblabcom, lib_func);
if code ~= 0
    if verbose
        rtc_message( code, lib_func);
    end
end
end

function RTCend( verbose )
%
%
%
% loadLibLabcom() is called in RTCstart.

%
% DllClass int RTCEnd( void )
lib_func = 'RTCEnd';
code = calllib( 'librttrans', lib_func);
if code ~= 0
    if verbose
        rtc_message(code, lib_func);
    end
end
end

function ret_mes = rtc_message( code , func_name)

%
% DllClass const char* RTCerrorMessage( int code)
lib_func = 'RTCerrorMessage';
mes = calllib( 'librttrans', lib_func, code);

if nargout > 0 
    ret_mes = mes;
else
    if nargin < 2 
        fprintf('Error from DLL ! %d : %s\n', code, mes);
    else
        fprintf('Error from DLL( %s ) ! %d : %s\n', func_name, code, mes);
    end
end
end
