function tvMonitor( diag_name, channel_number, varargin)
% Monitor image channel with RTTRansd.
%   tvMonitor( diag_name, channel_number, varargin) 
%
%       diag_name(text)         : name of diagnostics
%       channel_number(numeric) : channel number
%
% OPTIONS :
%         'thinning',thinnings(numeric)
%                           : transfer thinning rate to 1/N (default: 1)
%         'udp',udp_no      : number of the UDP to recieve data
%                             (default: defined system)
%
% Example :
%
%   >> tvMonitor('TSCCD',1);
%
% Last update : 2020-05-20
%
% See also
%     waveMonitor
%

if nargin < 2
    disp('<Error: the argument is short.>');
    disp('Type "help tvMonitor".');
    return;
end

ip = inputParser;

ip.addRequired('diag_name', @ischar);
ip.addRequired('channel_number', @(x)isnumeric(x) && isscalar(x) && x > 0);

ip.addParameter('udp', 0, @(x) isnumeric(x) && (x >= 0)); 
ip.addParameter('thinning', 1, @(x) isnumeric(x) && (x > 0)); 
ip.addParameter('host', '', @ischar); 

ip.parse(diag_name, channel_number,varargin{:});

thinnings = ip.Results.thinning;
udp_no = ip.Results.udp;
host_name = ip.Results.host;
verbose = true;
while true
    code = LRTCstart( verbose );
    if code ~= 0 
        break;
    end
    packet_size = 0;
    [hdl_ptr,code] = LRTCopen(diag_name, channel_number, thinnings, udp_no, ...
                    packet_size, host_name, verbose);
    if code ~= 0 
        break;
    end

    while true
        % TODO setInf test only
        code = LRTCsetInf( hdl_ptr, thinnings, verbose );
        if code ~= 0 
            break;
        end
        
%        [code, ~, value_len, rec_len, ~, status] = ...
%            LRTgetChannelParam( hdl_ptr, verbose );
%        [code, pixel_dipth, fr_rate, fr_width, fr_height, rec_len, packet_size, status] = ...
        [code, pixel_dipth, ~, fr_width, fr_height,~,~,~,buf_type ] = ...
            LRTCgetChannelParam( hdl_ptr, verbose );        
        if code ~= 0 
            break;
        end
        [code, image_type ] = ...
            LRTgetChParam( hdl_ptr, 'ImageType', verbose );
        if code ~= 0 
            break;
        end
        if strcmp(image_type,'IllegalParameterName' )
            image_type = 'unkown';
        end
        img=1; % RGB
        if strncmp(image_type,'GRAY', 4 ) || strncmp(image_type,'GREY', 4 )
            img=0; % GRAY
        elseif strcmp(image_type,'YUV422')
            img=2;
        elseif strcmp(image_type,'YUY2') 
            img=3; 
        end
        % TODO
        fprintf('dipth=%d,width=%d,height=%d,buf_type=%s,image=%s\n', ...
            pixel_dipth,fr_width,fr_height,buf_type,image_type);
        
        hf=figure;
        % figure(hf);        
        % if 17 > pixel_dipth 2020-05
        if 0 == img
            % 2020-05 Colormap->colormap
            colormap(gray(double(2^pixel_dipth)));
        end
        stat_ptr = libpointer('int32Ptr',zeros(1,'int32'));
        shot_ptr = libpointer('int32Ptr',zeros(1,'int32'));
        sub_ptr = libpointer('int32Ptr',zeros(1,'int32'));
        blk_ptr = libpointer('int32Ptr',zeros(1,'int32'));
        head_ptr = libpointer('int64Ptr',zeros(1,'int64'));
        tail_ptr = libpointer('int64Ptr',zeros(1,'int64'));
        buf_ptr = libpointer([buf_type 'Ptr'],zeros(fr_width,fr_height));
        len_ptr = libpointer('int32Ptr',zeros(1,'int32'));

        disp('Terminate when "q" or "Q" is entered.');
        while 1
            code = LRTCgetChannelData(hdl_ptr, stat_ptr, shot_ptr, sub_ptr, ...
            blk_ptr, head_ptr, tail_ptr, buf_ptr, len_ptr, verbose );
            if code == 0 
                fprintf('blk=%d %d\n',blk_ptr.Value',stat_ptr.Value);
                if stat_ptr.Value == 8          % Acquire status
                    % figure(hf);
                    % if 17 > pixel_dipth 2020-05-20
                    if 0 == img
                        imshow(buf_ptr.Value');
                    elseif(2 == img || 3 == img )
                        imshow(yuv2rgb(buf_ptr,fr_width,fr_height,img-2),'Border','tight');
                    else
                        imshow(bgrx2rgb(buf_ptr,fr_width,fr_height));
                    end
                    title([diag_name ' CH#' num2str(channel_number)]);
                elseif stat_ptr.Value == 3      % Setup status
                    [code, pixel_dipth, ~, fr_width, fr_height,~,~,~,buf_type ] = ...
                    LRTCgetChannelParam( hdl_ptr, verbose );        
                    if code ~= 0 
                        break;
                    end
                    buf_ptr = libpointer([buf_type 'Ptr'],zeros(fr_width,fr_height));
                    % if 17 > pixel_dipth 2020-05
                    if 0 == img
                        colormap(gray(double(2^pixel_dipth)));
                    end
                else
                    pause(0.4);
                end
            elseif code == 14  % 14 is timeout
                pause(0.1);
            elseif code == 11
                continue;
            else
                break;          % other error
            end
            if strcmpi(get(hf,'currentcharacter'),'q')
                break;
            end
        end
        close(hf);
        break;
    end    
    LRTCclose(hdl_ptr, verbose);
    break;
   
end
LRTCend(verbose);
if verbose
    disp('End.');
end
end

function [hdl_ptr,code] = LRTCopen( diag_name, channel_number, thinnings, udp_no, packet_size, host, verbose )
%
%
% Last update : 2020-05-20 add host, 
%
% loadLibLabcom() is called in LRTstart.LRTCameraOpen2
%

% TODO comment out if external func.
%if ~exist('verbose','var')
%    verbose = true;
%end
%
%  DllClass	int	LRTCameraOpen(char *diagname, int chno, LRTINF inf, ...
%                        void *hnd, int udpp, int packetsize);
%  DllClass	int	LRTCameraOpen2(char *diagname, int chno, LRTINF inf, ...
%                        void *hnd, int udpp, int packetsize, const char* host);
inf = libstruct('lrtinf');
inf.thinning = thinnings;
hdl_ptr = libpointer('voidPtr',zeros(1,256,'int8'));
lib_func = 'LRTCameraOpen2';
%[code,diag_name,hdl_ptr]
code = calllib( 'librttrans', lib_func, diag_name, channel_number, ...
    inf, hdl_ptr, udp_no, packet_size, host);
if code ~= 0
    if verbose
        lrt_message(code, lib_func);
    end
end
end

function code = LRTCclose( hdl_ptr, verbose )
%
%
%
% loadLibLabcom() is called in LRTstart.

%
%  DllClass	int	LRTCameraClose( void *hnd);
lib_func = 'LRTCameraClose';
% [code, hdl_ptr]
code = calllib( 'librttrans', lib_func, hdl_ptr);
if code ~= 0
    if verbose
        lrt_message(code, lib_func);
    end
end
end
function code = LRTget_last_error()
%
%
%

% [code]
code = calllib( 'librttrans', 'LRTget_last_error');
end
function [code, pixel_dipth, fr_rate, fr_width, fr_height, rec_len, packet_size, status, buf_type] = ...
    LRTCgetChannelParam( hdl_ptr, verbose )
%
%
% samplingInterval: 0
%         valueLen: 0
%           recLen: 0
%       packetSize: 0
%
% loadLibLabcom() is called in LRTCstart.


%
% DllClass	int	LRTCameraGetChannelParam(void *hnd, int *status, LRTCAMERAPARAM *prm);
%     is NG.
% DllClass	int	LRTCameraGetChannelParam2(void *hnd, int *status,
%				int* dipth, int* rate, int* width, int* height, int* rec_len,int* packet_size);
dipth_ptr = libpointer('int32Ptr',zeros(1,'int32'));
rate_ptr = libpointer('int32Ptr',zeros(1,'int32'));
width_ptr = libpointer('int32Ptr',zeros(1,'int32'));
height_ptr = libpointer('int32Ptr',zeros(1,'int32'));
rec_l_ptr = libpointer('int32Ptr',zeros(1,'int32'));
psize_ptr = libpointer('int32Ptr',zeros(1,'int32'));
stat_ptr = libpointer('int32Ptr',zeros(1,'int32'));
lib_func = 'LRTCameraGetChannelParam2';
code=calllib( 'librttrans', lib_func, hdl_ptr, stat_ptr, ...
    dipth_ptr, rate_ptr, width_ptr, height_ptr, rec_l_ptr, psize_ptr);

if code ~= 0
    if verbose
        lrt_message(code, lib_func);
    end
end

status = stat_ptr.Value;
pixel_dipth = dipth_ptr.Value;
fr_rate = rate_ptr.Value;
fr_width = width_ptr.Value;
fr_height = height_ptr.Value;
rec_len = rec_l_ptr.Value;
packet_size = psize_ptr.Value;

if pixel_dipth < 9
   buf_type = 'uint8';
elseif pixel_dipth < 17
   buf_type = 'uint16';
else
   buf_type = 'uint32';
end

end

function code = LRTCgetChannelData(hdl_ptr, stat_ptr, shot_ptr, sub_ptr, ...
    blk_ptr, head_ptr, tail_ptr, buf_ptr, len_ptr, verbose )
%
%
%
% loadLibLabcom() is called in LRTstart.
%
% stat_ptr = libpointer('int32Ptr',zeros(1,'int32'));
% shot_ptr = libpointer('int32Ptr',zeros(1,'int32'));
% sub_ptr = libpointer('int32Ptr',zeros(1,'int32'));
% blk_ptr = libpointer('int32Ptr',zeros(1,'int32'));
% head_ptr = libpointer('int64Ptr',zeros(1,'int64'));
% tail_ptr = libpointer('int64Ptr',zeros(1,'int64'));
% buf_ptr = libpointer('voidPtr',....);
% len_ptr = libpointer('int32Ptr',zeros(1,'int32'));

%
% DllClass	int	LRTGetChannelData(void *hnd, int *status, ...
%	int *shotno, int *subshot,int *blkno, int64_t *head, int64_t *tail, ...
%   void *buf, int *len);
lib_func = 'LRTCameraGetChannelData';
%[ code,hdl_ptr,stat_ptr,shot_ptr,sub_ptr,blk_ptr,head_ptr,tail_ptr,buf_ptr,len_ptr]
code = calllib( 'librttrans', lib_func, hdl_ptr, stat_ptr, ...
    shot_ptr, sub_ptr, blk_ptr, head_ptr, tail_ptr, buf_ptr, len_ptr );
if code == 14
    return;    
elseif code == 11 % add 2020-05
    return;
elseif code ~= 0
    if verbose
        lrt_message( code, lib_func);
    end
end
end

function code = LRTCsetInf( hdl_ptr, thinnings , verbose)
%
%
%
% loadLibLabcom() is called in LRTstart.

%
%  DllClass	int	LRTSetInf(void *hnd,  LRTINF inf);
lib_func = 'LRTCameraSetInf';
inf = libstruct('lrtinf');
inf.thinning = thinnings;
%[code,hdl_ptr]
code = calllib( 'librttrans', lib_func, hdl_ptr, inf);
if code ~= 0
    if verbose
        lrt_message( code, lib_func);
    end
end
end

function code = LRTCstart( verbose )
%
%
%

liblabcom = 'librttrans';
if ~loadLibLabcom( liblabcom, verbose )
    code = -9;
    return;
end
%
% DllClass int LRTStart( void )
lib_func = 'LRTCameraStart';
code = calllib( liblabcom, lib_func);
if code ~= 0
    if verbose
        lrt_message( code, lib_func);
    end
end
end

function LRTCend( verbose )
%
%
%
% loadLibLabcom() is called in LRTstart.

%
% DllClass int LRTEnd( void )
lib_func = 'LRTCameraEnd';
code = calllib( 'librttrans', lib_func);
if code ~= 0
    if verbose
        lrt_message(code, lib_func);
    end
end
end
function dst = bgrx2rgb(src_ptr, w, h)
%
%
%

dst_ptr = libpointer('uint8Ptr',zeros(h,w,3,'uint8'));
calllib( 'librttrans', 'LRTbgrx2rgb4matlab', dst_ptr, src_ptr, w,h);
dst = reshape(dst_ptr.Value,h,w,3);

end

function dst = yuv2rgb(src_ptr, w, h, ytype)
%
%
%

dst_ptr = libpointer('uint8Ptr',zeros(h,w,3,'uint8'));
calllib( 'librttrans', 'LRTyuv2rgb4matlab', dst_ptr, src_ptr, w,h, ytype);
dst = reshape(dst_ptr.Value,h,w,3);

end

function ret_mes = lrt_message( code , func_name)

switch code
    case	0
        mes = 'No Error';        
    case	1
        mes = 'Diag Name Not Found';
	case	2
        mes = 'Channel Number Over Range';
	case	3
        mes = 'Connect to Index Server Failed';
	case	4
        mes = 'INDEXSERVERNAME Not Defined';
	case	5
        mes = 'Error in SQL statement';
	case	6
        mes = 'Server IP Address Not Found';
	case	7
        mes = 'Connect to RT Server Failed';
	case	8
        mes = 'Error in Sending Command';
	case	9
        mes = 'Error in Receiving Reply';
	case	10
        mes = 'Already Closed Connection';
	case	11
        mes = 'Error in Receiveing Data';
	case	12
        mes = 'Invalid Socket';
	case	13
        mes = 'Error in Making Scoket';
	case	14
        mes = 'Time Out';
	case	15
        mes = 'Wrong Paramter Number';
	case	16
        mes = 'Not Wave Data Channel';
	case	17
        mes = 'Not Image Data Channel';
	case	18
        mes = 'Illegal handle';
	case	55
        mes = 'No diag name in database';

	case	200
        mes = 'Illegal Command (Please Contact to LABCOM)';
	case	201
        mes = 'Wrong Channel Number';
	case	202
        mes = 'Wrong Packet Size';
	case	203
        mes = 'Wrong Thinning Rate';
	case	204
        mes = 'Send Command to Unopened Service';
	case	205
        mes = 'Send Open Command Repeatly';
	case	207
        mes = 'Send Stop Command Repeatly';
	case	208
        mes = 'Send Start Command Repeatly';
	case	209
        mes = 'Connection Refused by Limit';
    otherwise
        mes = ['code = ' num2str(code)];
end
if nargout > 0 
    ret_mes = mes;
else
    if nargin < 2 
        fprintf('Error from DLL ! %d : %s\n', code, mes);
    else
        last = LRTget_last_error();
        fprintf('Error from DLL( %s ) ! %d : %s(%d)\n', func_name, code, mes, last);
    end
end
end
function [code, pvalue] = LRTgetChParam(hdl_ptr, pkeyword, verbose )
%
%
%
% loadLibLabcom() is called in LRTstart.
%

%
% DllClass	int		LRTGetChParam(void *hnd, char *Key, char *Value);
lib_func = 'LRTGetChParam';
%[ code,hdl_ptr,stat_ptr,shot_ptr,sub_ptr,blk_ptr,head_ptr,tail_ptr,buf_ptr,len_ptr]
[code, ~, ~, pvalue] = calllib( 'librttrans', lib_func, hdl_ptr, pkeyword, blanks(128) );
if code ~= 0
    if verbose
        lrt_message( code, lib_func);
    end
end
end