function wave3Monitor( diag_name, channel_number, varargin)
% Monitor wave channel with RTTRansd2(transfer protocol ver.3).
%
%   wave3Monitor( diag_name, channel_number[, control_port], varargin) 
%
%       diag_name(text)     : name of diagnostics
%       channel_number(numeric)
%                           : channel number
%       control_port(numeric)
%                           : control port number
%
% OPTIONS :
%         'thinning',thinnings(numeric)
%                           : transfer thinning rate to 1/N (default:1)
%         'x_points',points(numeric)
%                           : number of points with X-axis (default:5000)
%         'udp' ,logical    : availability of UDP.  (default: false)
%         'host',hostname(text)
%                           : monitor from a direct server
%
% Examples :
%
%   >> wave3Monitor('CDP',2);
%
%   >> wave3Monitor('CDP',2,'thinning',10);
%
%   >> wave3Monitor('CDP',2,'x_points',10000);
%
% Note : 'nsum' option is not supported.
%
% Last update : 2020-06-30
%
% See also
%     tv3Monitor

if nargin < 2
    disp('<Error: the argument is short.>');
    disp('Type "help wave3Monitor".');
    return;
end

ip = inputParser;

validScalarPosNum = @(x) isnumeric(x) && isscalar(x) && (x > 0);
addRequired(ip,'diag_name', @ischar);
addRequired(ip,'channel_number',validScalarPosNum);
addOptional(ip,'control_port',-1,validScalarPosNum);

addParameter(ip,'udp', 0, @(x)islogical(x));
addParameter(ip,'thinning', 1, @(x)isnumeric(x) && x > 0);
addParameter(ip,'x_points', 5000, @(x)isnumeric(x) && x > 0);
addParameter(ip,'host', '', @ischar);

parse(ip,diag_name,channel_number,varargin{:});

PLOT_POINTS = ip.Results.x_points;
thinnings = ip.Results.thinning;
udp_use   = ip.Results.udp;
host_name = ip.Results.host;
ctrl_port = ip.Results.control_port;
verbose = true;
if udp_use
    packet_size = 0;
else
    packet_size = -1;
end
channel_list= num2str(channel_number);

while true
    code = RTCstart( verbose );
    if code ~= 0 
        break;
    end
    
    thin_type = 0;
	timeout = 1000;
    desc = RTCopen(diag_name, host_name, channel_list, ctrl_port, packet_size, thinnings, thin_type, verbose);
    if desc < 0 
        break;
    end
    while true
        
        [status, ~, value_len, rec_len] = RTCgetChannelParam( desc, channel_number, verbose );
        if status < 0 
            break;
        end
        
        [code, image_type ] = ...
            RTCgetChParam( desc, channel_number, 'ImageType', verbose );
        if code ~= 0 
            break;
        end
        if strcmp(image_type,'IllegalParameterName' )
            image_type = 'unkown';
        end
        fprintf('Format is %s (valueLen:%d)\n', image_type, value_len);

        if value_len < 9
            buf_type = 'int8';
            bytesize = 1;
        elseif value_len < 17
            buf_type = 'int16';
            bytesize = 2;
        elseif value_len < 33
            if strcmp(image_type ,'FLT32') 
                buf_type = 'single';
            else
                buf_type = 'int32';
            end
            bytesize = 4;
        else
            if strcmp(image_type ,'FLT64') 
                buf_type = 'double';
            else
                buf_type = 'int64';
            end
            bytesize = 8;
        end
        buf_len = rec_len/bytesize;
        if PLOT_POINTS < buf_len
            PLOT_POINTS = (int32(buf_len/PLOT_POINTS)+2)*PLOT_POINTS;
        end
        hf=figure; title(diag_name);
        stat_ptr = libpointer('int32Ptr',zeros(1,'int32'));
        shot_ptr = libpointer('int32Ptr',zeros(1,'int32'));
        sub_ptr = libpointer('int32Ptr',zeros(1,'int32'));
        blk_ptr = libpointer('int32Ptr',zeros(1,'int32'));
        head_ptr = libpointer('int64Ptr',zeros(1,'int64'));
        tail_ptr = libpointer('int64Ptr',zeros(1,'int64'));
        buf_ptr = libpointer([buf_type 'Ptr'],zeros(1,buf_len));
        len_ptr = libpointer('int32Ptr',zeros(1,'int32'));

        plot_buf = zeros(1,PLOT_POINTS,buf_type);
        en_p=0;
        current_blk = 0;
        code = RTCtransfer( desc, 1, verbose );
        if code ~= 0 
            break;
        end
        
        
        disp('Terminate when "q" or "Q" is entered.');
        while 1
            code = RTCgetChannelData(desc, channel_number, timeout, ...
            stat_ptr, shot_ptr, sub_ptr, ...
            blk_ptr, head_ptr, tail_ptr, buf_ptr, len_ptr, verbose );
            if code == 0 
                if stat_ptr.Value == 8      % Acquire status
                    buf_len = len_ptr.Value/bytesize;
                    en_p = en_p + buf_len;
                    if current_blk == blk_ptr.Value
                        continue;
                    elseif (current_blk+1) ~= blk_ptr.Value
                        if current_blk ~= 0
                            disp(['Illegal block :' num2str(blk_ptr.Value)]);
                        end
                    end
                    current_blk = blk_ptr.Value;
            
                    if en_p <= PLOT_POINTS
                        plot_buf(en_p-buf_len+1:en_p) = buf_ptr.Value(1:buf_len);
                    else
                        mv_l = en_p - PLOT_POINTS;
                        plot_buf(1:PLOT_POINTS-mv_l) = plot_buf(mv_l+1:PLOT_POINTS);
                        plot_buf(PLOT_POINTS-buf_len+1:PLOT_POINTS) = buf_ptr.Value(1:buf_len);
                        en_p = PLOT_POINTS;
                    end
                else
                    pause(0.4);
                end
            elseif code == 1
                 pause(0.01);
            elseif code < 0
                break;
            else
                continue;
            end
            figure(hf);
            plot(plot_buf);title([diag_name ' CH#' num2str(channel_number)]);
            drawnow;     % for R2014b 2015-02-17 
            if strcmpi(get(hf,'currentcharacter'),'q')
                close(hf);
                break
            end

        end
        break;
    end    
    RTCclose(desc, verbose);
    break;
   
end
RTCend(verbose);
disp('End');
end

function desc = RTCopen( diag_name, server, channel_list, control_port, ...
                                   packet_size, thinnings, thin_type, verbose )
%
%
%
% loadLibLabcom() is called in RTCstart.
%

% TODO comment out if external func.
%if ~exist('verbose','var')
%    verbose = true;
%end
%
% DllClass int RTCopen( char* diag, char* server, char* channel_list, 
%					   int control_port, int packet_size, int thinning, int thin_type );
lib_func = 'RTCopen';
desc = calllib( 'librttrans', lib_func, diag_name, server, channel_list, ...
        control_port, packet_size, thinnings, thin_type);
if desc < 0
    if verbose
        rtc_message(desc, lib_func);
    end
end
end

function code = RTCclose( desc, verbose )
%
%
%
% loadLibLabcom() is called in RTCstart.

%
% DllClass int RTCclose( int desc);
lib_func = 'RTCclose';
% [code]
code = calllib( 'librttrans', lib_func, desc);
if code ~= 0
    if verbose
        rtc_message(code, lib_func);
    end
end
end

function [status, samp_interval, value_len, rec_len ] = ...
    RTCgetChannelParam( desc, chno, verbose )
%
%
% samplingInterval: 0
%         valueLen: 0
%           recLen: 0
%
% loadLibLabcom() is called in RTCstart.

%
sampi_ptr = libpointer('int32Ptr',zeros(1,'int32'));
val_l_ptr = libpointer('int32Ptr',zeros(1,'int32'));
rec_l_ptr = libpointer('int32Ptr',zeros(1,'int32'));
lib_func = 'RTCgetChannelParam2';
status=calllib( 'librttrans', lib_func, desc, chno, ...
    sampi_ptr, val_l_ptr, rec_l_ptr);

if status < 0
    if verbose
        rtc_message(status, lib_func);
    end
end

samp_interval = sampi_ptr.Value;
value_len = val_l_ptr.Value;
rec_len = rec_l_ptr.Value;
end

function code = RTCgetChannelData(desc, chno, timeout, stat_ptr, shot_ptr, sub_ptr, ...
    blk_ptr, head_ptr, tail_ptr, buf_ptr, len_ptr, verbose )
%
%
%
% loadLibLabcom() is called in RTCstart.
%
% stat_ptr = libpointer('int32Ptr',zeros(1,'int32'));
% shot_ptr = libpointer('int32Ptr',zeros(1,'int32'));
% sub_ptr = libpointer('int32Ptr',zeros(1,'int32'));
% blk_ptr = libpointer('int32Ptr',zeros(1,'int32'));
% head_ptr = libpointer('int64Ptr',zeros(1,'int64'));
% tail_ptr = libpointer('int64Ptr',zeros(1,'int64'));
% buf_ptr = libpointer('voidPtr',....);
% len_ptr = libpointer('int32Ptr',zeros(1,'int32'));

%
% int RTCgetChannelData( int desc, int chno, int timeout,
%                                 int *status, int *shotno, int *subshot,
%								 int *blkno, int64_t *headno, int64_t *tailno, char* buf, int *len )
lib_func = 'RTCgetChannelData';
code = calllib( 'librttrans', lib_func, desc, chno, timeout, stat_ptr, ...
    shot_ptr, sub_ptr, blk_ptr, head_ptr, tail_ptr, buf_ptr, len_ptr );
if code == 14
    return;
elseif code < 0
    if verbose
        rtc_message( code, lib_func);
    end
end
end

function [code, pvalue] = RTCgetChParam(desc, chno, pkeyword, verbose )
%
%
%
% loadLibLabcom() is called in RTCstart.
%

%
% DllClass	int	RTCgetChannelKeyParam( int desc, int chno, const char*key, char* buffer, int buffer_size);
lib_func = 'RTCgetChannelKeyParam';
[code, ~, pvalue] = calllib( 'librttrans', lib_func, desc, chno, pkeyword, blanks(128), 128 );
if code ~= 0
    if verbose
        rtc_message( code, lib_func);
    end
end
end

function code = RTCtransfer( desc, action, verbose )
%
%
%
% loadLibLabcom() is called in RTCstart.

%
% DllClass int RTCtransfer( int desc, int action);
lib_func = 'RTCtransfer';
code = calllib( 'librttrans', lib_func, desc, action);
if code ~= 0
    if verbose
        rtc_message(code, lib_func);
    end
end
end

function code = RTCstart( verbose )
%
%
%

liblabcom = 'librttrans';
if ~loadLibLabcom( liblabcom, verbose )
    code = -9;
    return;
end
%
% DllClass int RTCStart( void )
lib_func = 'RTCStart';
code = calllib( liblabcom, lib_func);
if code ~= 0
    if verbose
        rtc_message( code, lib_func);
    end
end
end

function RTCend( verbose )
%
%
%
% loadLibLabcom() is called in RTCstart.

%
% DllClass int RTCEnd( void )
lib_func = 'RTCEnd';
code = calllib( 'librttrans', lib_func);
if code ~= 0
    if verbose
        rtc_message(code, lib_func);
    end
end
end

function ret_mes = rtc_message( code , func_name)

%
% DllClass const char* RTCerrorMessage( int code)
lib_func = 'RTCerrorMessage';
mes = calllib( 'librttrans', lib_func, code);

if nargout > 0 
    ret_mes = mes;
else
    if nargin < 2 
        fprintf('Error from DLL ! %d : %s\n', code, mes);
    else
        fprintf('Error from DLL( %s ) ! %d : %s\n', func_name, code, mes);
    end
end
end
