function waveMonitor( diag_name, channel_number, varargin)
% Monitor wave channel with RTTRansd.
%   waveMonitor( diag_name, channel_number, varargin) 
%
%       diag_name(text)     : name of diagnostics
%       channel_number(numeric)
%                           : channel number
%
% OPTIONS :
%         'thinning',thinnings(numeric)
%                           : transfer thinning rate to 1/N (default:1)
%         'x_points',points(numeric)
%                           : number of points with X-axis (default:5000)
%         'udp',udp_no      : number of the UDP to recieve data
%                             (default: defined system)
%         'host',hostname(text)
%                           : monitor from a direct server
%
% Examples :
%
%   >> waveMonitor('CDP',2);
%
%   >> waveMonitor('CDP',2,'thinning',10);
%
%   >> waveMonitor('CDP',2,'x_points',10000);
%
% Note : 'nsum' option is not supported.
%
% Last update : 2015-02-17
%
% See also
%     tvMonitor

if nargin < 2
    disp('<Error: the argument is short.>');
    disp('Type "help waveMonitor".');
    return;
end

ip = inputParser;

ip.addRequired('diag_name', @ischar);
ip.addRequired('channel_number', @(x)isnumeric(x) && isscalar(x) && x > 0);

ip.addParameter('udp', 0, @(x)isnumeric(x) && x >= 0);
ip.addParameter('thinning', 1, @(x)isnumeric(x) && x > 0);
ip.addParameter('x_points', 5000, @(x)isnumeric(x) && x > 0);
ip.addParameter('host', '', @ischar);

ip.parse(diag_name, channel_number,varargin{:});

PLOT_POINTS = ip.Results.x_points;
thinnings = ip.Results.thinning;
udp_no = ip.Results.udp;
host_name = ip.Results.host;

verbose = true;
while true
    code = LRTstart( verbose );
    if code ~= 0 
        break;
    end
    packet_size = 0;
    [hdl_ptr,code] = LRTopen(diag_name, channel_number, thinnings, udp_no, packet_size, host_name, verbose);
    if code ~= 0 
        break;
    end

    while true
        % TODO setInf test only
        code = LRTsetInf( hdl_ptr, thinnings, verbose );
        if code ~= 0 
            break;
        end
        
        [code, ~, value_len, rec_len, ~, status] = ...
            LRTgetChannelParam( hdl_ptr, verbose );
        if code ~= 0 
            break;
        end
        
        [code, image_type ] = ...
            LRTgetChParam( hdl_ptr, 'ImageType', verbose );
        if code ~= 0 
            break;
        end
        if strcmp(image_type,'IllegalParameterName' )
            image_type = 'unkown';
        end
        fprintf('Format is %s (valueLen:%d)\n', image_type, value_len);

        if value_len < 9
            buf_type = 'int8';
            bytesize = 1;
        elseif value_len < 17
            buf_type = 'int16';
            bytesize = 2;
        elseif value_len < 33
            if strcmp(image_type ,'FLT32') 
                buf_type = 'single';
            else
                buf_type = 'int32';
            end
            bytesize = 4;
        else
            if strcmp(image_type ,'FLT64') 
                buf_type = 'double';
            else
                buf_type = 'int64';
            end
            bytesize = 8;
        end
        buf_len = rec_len/bytesize;
        if PLOT_POINTS < buf_len
            PLOT_POINTS = (int32(buf_len/PLOT_POINTS)+2)*PLOT_POINTS;
        end
        hf=figure; title(diag_name);
        stat_ptr = libpointer('int32Ptr',zeros(1,'int32'));
        shot_ptr = libpointer('int32Ptr',zeros(1,'int32'));
        sub_ptr = libpointer('int32Ptr',zeros(1,'int32'));
        blk_ptr = libpointer('int32Ptr',zeros(1,'int32'));
        head_ptr = libpointer('int64Ptr',zeros(1,'int64'));
        tail_ptr = libpointer('int64Ptr',zeros(1,'int64'));
        buf_ptr = libpointer([buf_type 'Ptr'],zeros(1,buf_len));
        len_ptr = libpointer('int32Ptr',zeros(1,'int32'));

        plot_buf = zeros(1,PLOT_POINTS,buf_type);
        en_p=0;
        current_blk = 0;
        disp('Terminate when "q" or "Q" is entered.');
        while 1
            code = LRTgetChannelData(hdl_ptr, stat_ptr, shot_ptr, sub_ptr, ...
            blk_ptr, head_ptr, tail_ptr, buf_ptr, len_ptr, verbose );
            if code == 0 
                if stat_ptr.Value == 8      % Acquire status
                    buf_len = len_ptr.Value/bytesize;
                    en_p = en_p + buf_len;
                    if current_blk == blk_ptr.Value
                        continue;
                    elseif (current_blk+1) ~= blk_ptr.Value
                        if current_blk ~= 0
                            disp(['Illegal block :' num2str(blk_ptr.Value)]);
                        end
                    end
                    current_blk = blk_ptr.Value;
            
                    if en_p <= PLOT_POINTS
                        plot_buf(en_p-buf_len+1:en_p) = buf_ptr.Value(1:buf_len);
                    else
                        mv_l = en_p - PLOT_POINTS;
                        plot_buf(1:PLOT_POINTS-mv_l) = plot_buf(mv_l+1:PLOT_POINTS);
                        plot_buf(PLOT_POINTS-buf_len+1:PLOT_POINTS) = buf_ptr.Value(1:buf_len);
                        en_p = PLOT_POINTS;
                    end
                else
                    pause(0.4);
                end
            elseif code == 14
                 pause(0.01);
            elseif code == 11
                continue;
            else
                break;
            end
            figure(hf);
            plot(plot_buf);title([diag_name ' CH#' num2str(channel_number)]);
            drawnow;     % for R2014b 2015-02-17 
            if strcmpi(get(hf,'currentcharacter'),'q')
                close(hf);
                break
            end

        end
        break;
    end    
    LRTclose(hdl_ptr, verbose);
    break;
   
end
LRTend(verbose);
disp('End');
end

function [hdl_ptr,code] = LRTopen( diag_name, channel_number, thinnings, ...
    udp_no, packet_size, mhost, verbose )
%
%
%
% loadLibLabcom() is called in LRTstart.
%

% TODO comment out if external func.
%if ~exist('verbose','var')
%    verbose = true;
%end
%
%  DllClass	int	LRTOpen(char *diagname, int chno, LRTINF inf, ...
%                        void *hnd, int udpp, int packetsize);
%  DllClass	int LRTOpen2(char *diagname, int chno, LRTINF inf, ...
%                   void *hnd, int udpp, int packetsize, const char* host);

inf = libstruct('lrtinf');
inf.thinning = thinnings;
hdl_ptr = libpointer('voidPtr',zeros(1,256,'int8'));
if isempty(mhost) 
    %[code,diag_name,hdl_ptr]
    lib_func = 'LRTOpen';
    code = calllib( 'librttrans', lib_func, diag_name, channel_number, ...
        inf, hdl_ptr, udp_no, packet_size);
else
    %[code,diag_name,hdl_ptr,mhost]
    lib_func = 'LRTOpen2';
    code = calllib( 'librttrans', lib_func, diag_name, channel_number, ...
        inf, hdl_ptr, udp_no, packet_size, mhost);
end
if code ~= 0
    if verbose
        lrt_message(code, lib_func);
    end
end
end

function code = LRTclose( hdl_ptr, verbose )
%
%
%
% loadLibLabcom() is called in LRTstart.

%
%  DllClass	int	LRTClose( void *hnd);
lib_func = 'LRTClose';
% [code, hdl_ptr]
code = calllib( 'librttrans', lib_func, hdl_ptr);
if code ~= 0
    if verbose
        lrt_message(code, lib_func);
    end
end
end
function [code, samp_interval, value_len, rec_len, packet_size, status] = ...
    LRTgetChannelParam( hdl_ptr, verbose )
%
%
% samplingInterval: 0
%         valueLen: 0
%           recLen: 0
%       packetSize: 0
%
% loadLibLabcom() is called in LRTstart.

%
% DllClass	int	LRTGetChannelParam(void *hnd, int *status, LRTPARAM *prm);
%     is NG.
% DllClass	int	LRTGetChannelParam2(void *hnd, int *status,
%				int* samp_interval, int* value_len, int* rec_len,int* packet_size);
sampi_ptr = libpointer('int32Ptr',zeros(1,'int32'));
val_l_ptr = libpointer('int32Ptr',zeros(1,'int32'));
rec_l_ptr = libpointer('int32Ptr',zeros(1,'int32'));
psize_ptr = libpointer('int32Ptr',zeros(1,'int32'));
stat_ptr = libpointer('int32Ptr',zeros(1,'int32'));
lib_func = 'LRTGetChannelParam2';
code=calllib( 'librttrans', lib_func, hdl_ptr, stat_ptr, ...
    sampi_ptr, val_l_ptr, rec_l_ptr, psize_ptr);

if code ~= 0
    if verbose
        lrt_message(code, lib_func);
    end
end

status = stat_ptr.Value;
samp_interval = sampi_ptr.Value;
value_len = val_l_ptr.Value;
rec_len = rec_l_ptr.Value;
packet_size = psize_ptr.Value;
end

function code = LRTgetChannelData(hdl_ptr, stat_ptr, shot_ptr, sub_ptr, ...
    blk_ptr, head_ptr, tail_ptr, buf_ptr, len_ptr, verbose )
%
%
%
% loadLibLabcom() is called in LRTstart.
%
% stat_ptr = libpointer('int32Ptr',zeros(1,'int32'));
% shot_ptr = libpointer('int32Ptr',zeros(1,'int32'));
% sub_ptr = libpointer('int32Ptr',zeros(1,'int32'));
% blk_ptr = libpointer('int32Ptr',zeros(1,'int32'));
% head_ptr = libpointer('int64Ptr',zeros(1,'int64'));
% tail_ptr = libpointer('int64Ptr',zeros(1,'int64'));
% buf_ptr = libpointer('voidPtr',....);
% len_ptr = libpointer('int32Ptr',zeros(1,'int32'));

%
% DllClass	int	LRTGetChannelData(void *hnd, int *status, ...
%	int *shotno, int *subshot,int *blkno, int64_t *head, int64_t *tail, ...
%   void *buf, int *len);
lib_func = 'LRTGetChannelData';
%[ code,hdl_ptr,stat_ptr,shot_ptr,sub_ptr,blk_ptr,head_ptr,tail_ptr,buf_ptr,len_ptr]
code = calllib( 'librttrans', lib_func, hdl_ptr, stat_ptr, ...
    shot_ptr, sub_ptr, blk_ptr, head_ptr, tail_ptr, buf_ptr, len_ptr );
if code == 14
    return;
elseif code ~= 0
    if verbose
        lrt_message( code, lib_func);
    end
end
end

function [code, pvalue] = LRTgetChParam(hdl_ptr, pkeyword, verbose )
%
%
%
% loadLibLabcom() is called in LRTstart.
%

%
% DllClass	int		LRTGetChParam(void *hnd, char *Key, char *Value);
lib_func = 'LRTGetChParam';
%[ code,hdl_ptr,stat_ptr,shot_ptr,sub_ptr,blk_ptr,head_ptr,tail_ptr,buf_ptr,len_ptr]
[code, ~, ~, pvalue] = calllib( 'librttrans', lib_func, hdl_ptr, pkeyword, blanks(128) );
if code ~= 0
    if verbose
        lrt_message( code, lib_func);
    end
end
end

function code = LRTsetInf( hdl_ptr, thinnings , verbose)
%
%
%
% loadLibLabcom() is called in LRTstart.

%
%  DllClass	int	LRTSetInf(void *hnd,  LRTINF inf);
lib_func = 'LRTSetInf';
inf = libstruct('lrtinf');
inf.thinning = thinnings;
%[code,hdl_ptr]
code = calllib( 'librttrans', lib_func, hdl_ptr, inf);
if code ~= 0
    if verbose
        lrt_message( code, lib_func);
    end
end
end

function code = LRTstart( verbose )
%
%
%

liblabcom = 'librttrans';
if ~loadLibLabcom( liblabcom, verbose )
    code = -9;
    return;
end
%
% DllClass int LRTStart( void )
lib_func = 'LRTStart';
code = calllib( liblabcom, lib_func);
if code ~= 0
    if verbose
        lrt_message( code, lib_func);
    end
end
end

function LRTend( verbose )
%
%
%
% loadLibLabcom() is called in LRTstart.

%
% DllClass int LRTEnd( void )
lib_func = 'LRTEnd';
code = calllib( 'librttrans', lib_func);
if code ~= 0
    if verbose
        lrt_message(code, lib_func);
    end
end
end

function ret_mes = lrt_message( code , func_name)

switch code
    case	0
        mes = 'No Error';        
    case	1
        mes = 'Diag Name Not Found';
	case	2
        mes = 'Channel Number Over Range';
	case	3
        mes = 'Connect to Index Server Failed';
	case	4
        mes = 'INDEXSERVERNAME Not Defined';
	case	5
        mes = 'Error in SQL statement';
	case	6
        mes = 'Server IP Address Not Found';
	case	7
        mes = 'Connect to RT Server Failed';
	case	8
        mes = 'Error in Sending Command';
	case	9
        mes = 'Error in Receiving Reply';
	case	10
        mes = 'Already Closed Connection';
	case	11
        mes = 'Error in Receiveing Data';
	case	12
        mes = 'Invalid Socket';
	case	13
        mes = 'Error in Making Scoket';
	case	14
        mes = 'Time Out';
	case	15
        mes = 'Wrong Paramter Number';
	case	16
        mes = 'Not Wave Data Channel';
	case	17
        mes = 'Not Image Data Channel';
	case	18
        mes = 'Illegal handle';
	case	55
        mes = 'No diag name in database';

	case	200
        mes = 'Illegal Command (Please Contact to LABCOM)';
	case	201
        mes = 'Wrong Channel Number';
	case	202
        mes = 'Wrong Packet Size';
	case	203
        mes = 'Wrong Thinning Rate';
	case	204
        mes = 'Send Command to Unopened Service';
	case	205
        mes = 'Send Open Command Repeatly';
	case	206
        mes = 'Shared data on server is unusual. Please wait and retry.';
	case	207
        mes = 'Send Stop Command Repeatly';
	case	208
        mes = 'Send Start Command Repeatly';
	case	209
        mes = 'Connection Refused by Limit';
    otherwise
        mes = ['code = ' num2str(code)];
end
if nargout > 0 
    ret_mes = mes;
else
    if nargin < 2 
        fprintf('Error from DLL ! %d : %s\n', code, mes);
    else
        fprintf('Error from DLL( %s ) ! %d : %s\n', func_name, code, mes);
    end
end
end
